@@ -33,7 +33,7 @@ class Slicot(package.Package):
       libs = [['-lslicot']]
 
     if self.packagedir:
-      dirs = [self.packagedir]
+      dirs = [os.path.join(self.packagedir,'lib'),self.packagedir]
     else:
       dirs = self.GenerateGuesses('slicot',archdir)
 
@@ -42,8 +42,7 @@ class Slicot(package.Package):
 
   def DownloadAndInstall(self,conf,vars,slepc,petsc,archdir,prefixdir):
     externdir = slepc.CreateDir(archdir,'externalpackages')
-    builddir  = os.path.join(externdir,self.dirname)
-    self.Download(externdir,builddir,slepc.downloaddir)
+    builddir  = self.Download(externdir,slepc.downloaddir)
     libname = 'libslicot.a'
 
     # Configure
