@@ -1,15 +1,14 @@
 #
 #  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 #  SLEPc - Scalable Library for Eigenvalue Problem Computations
-#  Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
+#  Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
 #
 #  This file is part of SLEPc.
 #  SLEPc is distributed under a 2-clause BSD license (see LICENSE).
 #  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 #
 
-import os, sys, commands
-import log, package
+import os, sys, log, package
 
 class Sowing(package.Package):
 
@@ -20,17 +19,12 @@ class Sowing(package.Package):
     self.url          = 'https://bitbucket.org/petsc/pkg-sowing.git'
     self.ProcessArgs(argdb)
 
-  def Install(self,archdir,make):
+  def DownloadAndInstall(self,slepc,petsc,archdir):
     name = self.packagename.upper()
     self.log.NewSection('Installing '+name+'...')
 
     # Create externalpackages directory
-    externdir = os.path.join(archdir,'externalpackages')
-    if not os.path.exists(externdir):
-      try:
-        os.mkdir(externdir)
-      except:
-        self.log.Exit('ERROR: cannot create directory ' + externdir)
+    externdir = slepc.CreateDir(archdir,'externalpackages')
 
     # Check if source is already available
     builddir = os.path.join(externdir,'pkg-sowing')
@@ -41,13 +35,13 @@ class Sowing(package.Package):
       if url=='':
         url = self.url
       try:
-        result,output = commands.getstatusoutput('cd '+externdir+'&& git clone '+url)
+        result,output = self.RunCommand('cd '+externdir+'&& git clone '+url)
         self.log.write(output)
-      except RuntimeError, e:
-        self.log.Exit('Error cloning '+url+': '+str(e))
+      except RuntimeError as e:
+        self.log.Exit('Cannot clone '+url+': '+str(e))
 
     # Configure, build and install package
-    result,output = commands.getstatusoutput('cd '+builddir+'&& ./configure --prefix='+archdir+'&&'+make+'&&'+make+' install')
+    result,output = self.RunCommand('cd '+builddir+'&& ./configure --prefix='+archdir+'&&'+petsc.make+'&&'+petsc.make+' install')
     self.log.write(output)
 
     self.havepackage = True
