@@ -1,22 +1,22 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#if !defined(_SVDIMPL)
-#define _SVDIMPL
+#if !defined(SLEPCSVDIMPL_H)
+#define SLEPCSVDIMPL_H
 
 #include <slepcsvd.h>
 #include <slepc/private/slepcimpl.h>
 
-PETSC_EXTERN PetscBool SVDRegisterAllCalled;
-PETSC_EXTERN PetscErrorCode SVDRegisterAll(void);
-PETSC_EXTERN PetscLogEvent SVD_SetUp,SVD_Solve;
+SLEPC_EXTERN PetscBool SVDRegisterAllCalled;
+SLEPC_EXTERN PetscErrorCode SVDRegisterAll(void);
+SLEPC_EXTERN PetscLogEvent SVD_SetUp,SVD_Solve;
 
 typedef struct _SVDOps *SVDOps;
 
@@ -104,7 +104,7 @@ struct _p_SVD {
 
 #define SVDCheckSolved(h,arg) \
   do { \
-    if (h->state<SVD_STATE_SOLVED) SETERRQ1(PetscObjectComm((PetscObject)h),PETSC_ERR_ARG_WRONGSTATE,"Must call SVDSolve() first: Parameter #%d",arg); \
+    if ((h)->state<SVD_STATE_SOLVED) SETERRQ1(PetscObjectComm((PetscObject)(h)),PETSC_ERR_ARG_WRONGSTATE,"Must call SVDSolve() first: Parameter #%d",arg); \
   } while (0)
 
 #endif
@@ -190,8 +190,8 @@ PETSC_STATIC_INLINE PetscErrorCode SVDMatGetLocalSize(SVD svd,PetscInt *m,PetscI
   PetscFunctionReturn(0);
 }
 
-PETSC_INTERN PetscErrorCode SVDTwoSideLanczos(SVD,PetscReal*,PetscReal*,BV,BV,PetscInt,PetscInt);
-PETSC_INTERN PetscErrorCode SVDSetDimensions_Default(SVD);
-PETSC_INTERN PetscErrorCode SVDComputeVectors(SVD);
+SLEPC_INTERN PetscErrorCode SVDTwoSideLanczos(SVD,PetscReal*,PetscReal*,BV,BV,PetscInt,PetscInt);
+SLEPC_INTERN PetscErrorCode SVDSetDimensions_Default(SVD);
+SLEPC_INTERN PetscErrorCode SVDComputeVectors(SVD);
 
 #endif
