@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -29,6 +29,7 @@ PetscErrorCode EPSSetUp_LAPACK(EPS eps)
   eps->ncv = eps->n;
   if (eps->mpd) { ierr = PetscInfo(eps,"Warning: parameter mpd ignored\n");CHKERRQ(ierr); }
   if (!eps->which) { ierr = EPSSetWhichEigenpairs_Default(eps);CHKERRQ(ierr); }
+  if (eps->which==EPS_ALL && eps->inta!=eps->intb) SETERRQ(PetscObjectComm((PetscObject)eps),1,"This solver does not support interval computation");
   if (eps->balance!=EPS_BALANCE_NONE) { ierr = PetscInfo(eps,"Warning: balancing ignored\n");CHKERRQ(ierr); }
   if (eps->stopping!=EPSStoppingBasic) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"User-defined stopping test not supported");
   if (eps->extraction) { ierr = PetscInfo(eps,"Warning: extraction type ignored\n");CHKERRQ(ierr); }
@@ -100,13 +101,11 @@ PetscErrorCode EPSSetUp_LAPACK(EPS eps)
   } else {
     ierr = PetscInfo(eps,"Using slow explicit operator\n");CHKERRQ(ierr);
     ierr = STGetOperator(eps->st,&shell);CHKERRQ(ierr);
-    ierr = MatComputeExplicitOperator(shell,&OP);CHKERRQ(ierr);
-    ierr = MatDestroy(&shell);CHKERRQ(ierr);
+    ierr = MatComputeOperator(shell,MATDENSE,&OP);CHKERRQ(ierr);
+    ierr = STRestoreOperator(eps->st,&shell);CHKERRQ(ierr);
     ierr = MatDestroy(&Adense);CHKERRQ(ierr);
-    ierr = MatCreateRedundantMatrix(OP,0,PETSC_COMM_SELF,MAT_INITIAL_MATRIX,&Ar);CHKERRQ(ierr);
+    ierr = MatCreateRedundantMatrix(OP,0,PETSC_COMM_SELF,MAT_INITIAL_MATRIX,&Adense);CHKERRQ(ierr);
     ierr = MatDestroy(&OP);CHKERRQ(ierr);
-    ierr = MatConvert(Ar,MATSEQDENSE,MAT_INITIAL_MATRIX,&Adense);CHKERRQ(ierr);
-    ierr = MatDestroy(&Ar);CHKERRQ(ierr);
   }
 
   /* fill DS matrices */
@@ -138,8 +137,8 @@ PetscErrorCode EPSSolve_LAPACK(EPS eps)
 {
   PetscErrorCode ierr;
   PetscInt       n=eps->n,i,low,high;
-  PetscScalar    *array,*pX;
-  Vec            v;
+  PetscScalar    *array,*pX,*pY;
+  Vec            v,w;
 
   PetscFunctionBegin;
   ierr = DSSolve(eps->ds,eps->eigr,eps->eigi);CHKERRQ(ierr);
@@ -153,22 +152,38 @@ PetscErrorCode EPSSolve_LAPACK(EPS eps)
     ierr = BVGetColumn(eps->V,i,&v);CHKERRQ(ierr);
     ierr = VecGetOwnershipRange(v,&low,&high);CHKERRQ(ierr);
     ierr = VecGetArray(v,&array);CHKERRQ(ierr);
-    ierr = PetscMemcpy(array,pX+i*n+low,(high-low)*sizeof(PetscScalar));CHKERRQ(ierr);
+    ierr = PetscArraycpy(array,pX+i*n+low,high-low);CHKERRQ(ierr);
     ierr = VecRestoreArray(v,&array);CHKERRQ(ierr);
     ierr = BVRestoreColumn(eps->V,i,&v);CHKERRQ(ierr);
   }
   ierr = DSRestoreArray(eps->ds,DS_MAT_X,&pX);CHKERRQ(ierr);
 
+  /* left eigenvectors */
+  if (eps->twosided) {
+    ierr = DSVectors(eps->ds,DS_MAT_Y,NULL,NULL);CHKERRQ(ierr);
+    ierr = DSGetArray(eps->ds,DS_MAT_Y,&pY);CHKERRQ(ierr);
+    for (i=0;i<eps->ncv;i++) {
+      ierr = BVGetColumn(eps->W,i,&w);CHKERRQ(ierr);
+      ierr = VecGetOwnershipRange(w,&low,&high);CHKERRQ(ierr);
+      ierr = VecGetArray(w,&array);CHKERRQ(ierr);
+      ierr = PetscArraycpy(array,pY+i*n+low,high-low);CHKERRQ(ierr);
+      ierr = VecRestoreArray(w,&array);CHKERRQ(ierr);
+      ierr = BVRestoreColumn(eps->W,i,&w);CHKERRQ(ierr);
+    }
+    ierr = DSRestoreArray(eps->ds,DS_MAT_Y,&pY);CHKERRQ(ierr);
+  }
+
   eps->nconv  = eps->ncv;
   eps->its    = 1;
   eps->reason = EPS_CONVERGED_TOL;
   PetscFunctionReturn(0);
 }
 
-PETSC_EXTERN PetscErrorCode EPSCreate_LAPACK(EPS eps)
+SLEPC_EXTERN PetscErrorCode EPSCreate_LAPACK(EPS eps)
 {
   PetscFunctionBegin;
   eps->useds = PETSC_TRUE;
+  eps->hasts = PETSC_TRUE;
   eps->categ = EPS_CATEGORY_OTHER;
 
   eps->ops->solve          = EPSSolve_LAPACK;
