@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2018, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -27,7 +27,7 @@
    References:
 
        [1] "Single Vector Iteration Methods in SLEPc", SLEPc Technical Report
-           STR-2, available at http://slepc.upv.es.
+           STR-2, available at https://slepc.upv.es.
 */
 
 #include <slepc/private/epsimpl.h>                /*I "slepceps.h" I*/
@@ -38,6 +38,8 @@
 
 static PetscErrorCode EPSPowerFormFunction_Update(SNES,Vec,Vec,void*);
 static PetscErrorCode SNESLineSearchPostheckFunction(SNESLineSearch linesearch,Vec x,Vec y,Vec w,PetscBool *changed_y,PetscBool *changed_w,void *ctx);
+PetscErrorCode EPSSolve_Power(EPS);
+PetscErrorCode EPSSolve_TS_Power(EPS);
 
 typedef struct {
   EPSPowerShiftType shift_type;
@@ -49,6 +51,9 @@ typedef struct {
   PetscErrorCode    (*formFunctionA)(SNES,Vec,Vec,void*);
   void              *formFunctionActx;
   PetscErrorCode    (*formFunctionAB)(SNES,Vec,Vec,Vec,void*);
+  PetscInt          idx;  /* index of the first nonzero entry in the iteration vector */
+  PetscMPIInt       p;    /* process id of the owner of idx */
+  PetscReal         norm0; /* norm of initial vector */
 } EPS_POWER;
 
 PetscErrorCode EPSSetUp_Power(EPS eps)
@@ -57,7 +62,7 @@ PetscErrorCode EPSSetUp_Power(EPS eps)
   EPS_POWER      *power = (EPS_POWER*)eps->data;
   PetscBool      flg,istrivial;
   STMatMode      mode;
-  Mat            A,B;
+  Mat            A,B,P;
   Vec            res;
   PetscContainer container;
   PetscErrorCode (*formFunctionA)(SNES,Vec,Vec,void*);
@@ -92,8 +97,17 @@ PetscErrorCode EPSSetUp_Power(EPS eps)
     if (eps->nev>1) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Nonlinear inverse iteration cannot compute more than one eigenvalue");
     ierr = EPSSetWorkVecs(eps,4);CHKERRQ(ierr);
 
-    /* set up SNES solver */
-    if (!power->snes) { ierr = EPSPowerGetSNES(eps,&power->snes);CHKERRQ(ierr); }
+    /* Set up SNES solver */
+    /* If SNES was setup, we need to reset it so that it will be consistent with the current EPS */
+    if (power->snes) { ierr = SNESReset(power->snes);CHKERRQ(ierr); }
+    else { ierr = EPSPowerGetSNES(eps,&power->snes);CHKERRQ(ierr); }
+
+    /* For nonlinear solver (Newton), we should scale the initial vector back.
+       The initial vector will be scaled in EPSSetUp. */
+    if (eps->IS) {
+      ierr = VecNorm((eps->IS)[0],NORM_2,&power->norm0);CHKERRQ(ierr);
+    }
+
     ierr = EPSGetOperators(eps,&A,&B);CHKERRQ(ierr);
 
     ierr = PetscObjectQueryFunction((PetscObject)A,"formFunction",&formFunctionA);CHKERRQ(ierr);
@@ -118,7 +132,12 @@ PetscErrorCode EPSSetUp_Power(EPS eps)
     if (container) {
       ierr = PetscContainerGetPointer(container,&ctx);CHKERRQ(ierr);
     } else ctx = NULL;
-    ierr = SNESSetJacobian(power->snes,A,A,formJacobianA,ctx);CHKERRQ(ierr);
+    /* This allows users to compute a different preconditioning matrix than A.
+     * It is useful when A and B are shell matrices.
+     */
+    ierr = STPrecondGetMatForPC(eps->st,&P);CHKERRQ(ierr);
+    /* If users do not provide a matrix, we simply use A */
+    ierr = SNESSetJacobian(power->snes,A,P? P:A,formJacobianA,ctx);CHKERRQ(ierr);
     ierr = SNESSetFromOptions(power->snes);CHKERRQ(ierr);
     ierr = SNESGetLineSearch(power->snes,&linesearch);CHKERRQ(ierr);
     if (power->update) {
@@ -133,35 +152,70 @@ PetscErrorCode EPSSetUp_Power(EPS eps)
       } else power->formFunctionBctx = NULL;
     }
   } else {
-    ierr = EPSSetWorkVecs(eps,2);CHKERRQ(ierr);
+    if (eps->twosided) {
+      ierr = EPSSetWorkVecs(eps,3);CHKERRQ(ierr);
+    } else {
+      ierr = EPSSetWorkVecs(eps,2);CHKERRQ(ierr);
+    }
     ierr = DSSetType(eps->ds,DSNHEP);CHKERRQ(ierr);
     ierr = DSAllocate(eps->ds,eps->nev);CHKERRQ(ierr);
   }
+  /* dispatch solve method */
+  if (eps->twosided) {
+    if (power->nonlinear) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Nonlinear inverse iteration does not have two-sided variant");
+    if (power->shift_type == EPS_POWER_SHIFT_WILKINSON) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Two-sided variant does not support Wilkinson shifts");
+    eps->ops->solve = EPSSolve_TS_Power;
+  } else eps->ops->solve = EPSSolve_Power;
   PetscFunctionReturn(0);
 }
 
+/*
+   Find the index of the first nonzero entry of x, and the process that owns it.
+*/
+static PetscErrorCode FirstNonzeroIdx(Vec x,PetscInt *idx,PetscMPIInt *p)
+{
+  PetscErrorCode    ierr;
+  PetscInt          i,first,last,N;
+  PetscLayout       map;
+  const PetscScalar *xx;
+
+  PetscFunctionBegin;
+  ierr = VecGetSize(x,&N);CHKERRQ(ierr);
+  ierr = VecGetOwnershipRange(x,&first,&last);CHKERRQ(ierr);
+  ierr = VecGetArrayRead(x,&xx);CHKERRQ(ierr);
+  for (i=first;i<last;i++) {
+    if (PetscAbsScalar(xx[i-first])>10*PETSC_MACHINE_EPSILON) break;
+  }
+  ierr = VecRestoreArrayRead(x,&xx);CHKERRQ(ierr);
+  if (i==last) i=N;
+  ierr = MPI_Allreduce(&i,idx,1,MPIU_INT,MPI_MIN,PetscObjectComm((PetscObject)x));CHKERRQ(ierr);
+  if (*idx==N) SETERRQ(PetscObjectComm((PetscObject)x),1,"Zero vector found");CHKERRQ(ierr);
+  ierr = VecGetLayout(x,&map);CHKERRQ(ierr);
+  ierr = PetscLayoutFindOwner(map,*idx,p);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
 
 /*
    Normalize a vector x with respect to a given norm as well as the
-   sign of the first entry.
+   sign of the first nonzero entry (assumed to be idx in process p).
 */
-static PetscErrorCode Normalize(Vec x,PetscReal norm,PetscScalar *sign)
+static PetscErrorCode Normalize(Vec x,PetscReal norm,PetscInt idx,PetscMPIInt p,PetscScalar *sign)
 {
   PetscErrorCode    ierr;
-  PetscScalar       alpha = 1.0;
-  PetscMPIInt       rank;
+  PetscScalar       alpha=1.0;
+  PetscInt          first,last;
   PetscReal         absv;
   const PetscScalar *xx;
 
   PetscFunctionBegin;
-  ierr = MPI_Comm_rank(PetscObjectComm((PetscObject)x),&rank);CHKERRQ(ierr);
-  if (!rank) {
+  ierr = VecGetOwnershipRange(x,&first,&last);CHKERRQ(ierr);
+  if (idx>=first && idx<last) {
     ierr = VecGetArrayRead(x,&xx);CHKERRQ(ierr);
-    absv = PetscAbsScalar(*xx);
-    if (absv>10*PETSC_MACHINE_EPSILON) alpha = *xx/absv;
+    absv = PetscAbsScalar(xx[idx-first]);
+    if (absv>10*PETSC_MACHINE_EPSILON) alpha = xx[idx-first]/absv;
     ierr = VecRestoreArrayRead(x,&xx);CHKERRQ(ierr);
   }
-  ierr = MPI_Bcast(&alpha,1,MPIU_SCALAR,0,PetscObjectComm((PetscObject)x));CHKERRQ(ierr);
+  ierr = MPI_Bcast(&alpha,1,MPIU_SCALAR,p,PetscObjectComm((PetscObject)x));CHKERRQ(ierr);
   if (sign) *sign = alpha;
   alpha *= norm;
   ierr = VecScale(x,1.0/alpha);CHKERRQ(ierr);
@@ -204,7 +258,7 @@ static PetscErrorCode EPSPowerUpdateFunctionA(EPS eps,Vec x,Vec Ax)
     } else {
       ierr = MatMult(A,x,Ax);CHKERRQ(ierr);
     }
-  } else SETERRQ(PETSC_COMM_WORLD,PETSC_ERR_ARG_NULL,"Matrix A is required for an eigenvalue problem\n");CHKERRQ(ierr);
+  } else SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_NULL,"Matrix A is required for an eigenvalue problem");
   PetscFunctionReturn(0);
 }
 
@@ -218,7 +272,7 @@ static PetscErrorCode SNESLineSearchPostheckFunction(SNESLineSearch linesearch,V
   PetscFunctionBegin;
   ierr = SNESLineSearchGetSNES(linesearch,&snes);CHKERRQ(ierr);
   ierr = PetscObjectQuery((PetscObject)snes,"eps",(PetscObject *)&eps);CHKERRQ(ierr);
-  if (!eps) SETERRQ(PETSC_COMM_WORLD,PETSC_ERR_ARG_NULL,"No composed EPS");
+  if (!eps) SETERRQ(PetscObjectComm((PetscObject)snes),PETSC_ERR_ARG_NULL,"No composed EPS");
   oldx = eps->work[3];
   ierr = VecCopy(x,oldx);CHKERRQ(ierr);
   PetscFunctionReturn(0);
@@ -234,7 +288,7 @@ static PetscErrorCode EPSPowerFormFunction_Update(SNES snes,Vec x,Vec y,void *ct
 
   PetscFunctionBegin;
   ierr = PetscObjectQuery((PetscObject)snes,"eps",(PetscObject *)&eps);CHKERRQ(ierr);
-  if (!eps) SETERRQ(PETSC_COMM_WORLD,PETSC_ERR_ARG_NULL,"No composed EPS");
+  if (!eps) SETERRQ(PetscObjectComm((PetscObject)snes),PETSC_ERR_ARG_NULL,"No composed EPS");
   power = (EPS_POWER*)eps->data;
   Bx = eps->work[2];
   if (power->formFunctionAB) {
@@ -244,7 +298,7 @@ static PetscErrorCode EPSPowerFormFunction_Update(SNES snes,Vec x,Vec y,void *ct
     ierr = EPSPowerUpdateFunctionB(eps,x,Bx);CHKERRQ(ierr);
   }
   ierr = VecNorm(Bx,NORM_2,&bx);CHKERRQ(ierr);
-  ierr = Normalize(Bx,bx,NULL);CHKERRQ(ierr);
+  ierr = Normalize(Bx,bx,power->idx,power->p,NULL);CHKERRQ(ierr);
   ierr = VecAXPY(y,-1.0,Bx);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -275,7 +329,7 @@ static PetscErrorCode EPSPowerApply_SNES(EPS eps,Vec x,Vec y)
 static PetscErrorCode EPSPowerComputeInitialGuess_Update(EPS eps)
 {
   EPS            powereps;
-  Mat            A,B;
+  Mat            A,B,P;
   Vec            v1,v2;
   SNES           snes;
   DM             dm,newdm;
@@ -292,7 +346,7 @@ static PetscErrorCode EPSPowerComputeInitialGuess_Update(EPS eps)
   ierr = EPSSetProblemType(powereps,EPS_GNHEP);CHKERRQ(ierr);
   ierr = EPSSetWhichEigenpairs(powereps,EPS_TARGET_MAGNITUDE);CHKERRQ(ierr);
   ierr = EPSPowerSetNonlinear(powereps,PETSC_TRUE);CHKERRQ(ierr);
-  ierr = STSetType(powereps->st,STSINVERT);CHKERRQ(ierr);
+  ierr = STPrecondGetMatForPC(eps->st,&P);CHKERRQ(ierr);
   /* attach dm to initial solve */
   ierr = EPSPowerGetSNES(eps,&snes);CHKERRQ(ierr);
   ierr = SNESGetDM(snes,&dm);CHKERRQ(ierr);
@@ -304,6 +358,9 @@ static PetscErrorCode EPSPowerComputeInitialGuess_Update(EPS eps)
   ierr = EPSPowerGetSNES(powereps,&snes);CHKERRQ(ierr);
   ierr = SNESSetDM(snes,dm);CHKERRQ(ierr);
   ierr = EPSSetFromOptions(powereps);CHKERRQ(ierr);
+  if (P) {
+    ierr = STPrecondSetMatForPC(powereps->st,P);CHKERRQ(ierr);
+  }
   ierr = EPSSolve(powereps);CHKERRQ(ierr);
   ierr = BVGetColumn(eps->V,0,&v2);CHKERRQ(ierr);
   ierr = BVGetColumn(powereps->V,0,&v1);CHKERRQ(ierr);
@@ -319,10 +376,6 @@ static PetscErrorCode EPSPowerComputeInitialGuess_Update(EPS eps)
 
 PetscErrorCode EPSSolve_Power(EPS eps)
 {
-#if defined(SLEPC_MISSING_LAPACK_LAEV2)
-  PetscFunctionBegin;
-  SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"LAEV2 - Lapack routine is unavailable");
-#else
   PetscErrorCode      ierr;
   EPS_POWER           *power = (EPS_POWER*)eps->data;
   PetscInt            k,ld;
@@ -344,7 +397,8 @@ PetscErrorCode EPSSolve_Power(EPS eps)
   if (power->shift_type != EPS_POWER_SHIFT_CONSTANT) { ierr = STGetKSP(eps->st,&ksp);CHKERRQ(ierr); }
   if (power->nonlinear) {
     ierr = PetscObjectCompose((PetscObject)power->snes,"eps",(PetscObject)eps);CHKERRQ(ierr);
-    if (power->update) {
+    /* Compute an intial guess only when users do not provide one */
+    if (power->update && !eps->nini) {
       ierr = EPSPowerComputeInitialGuess_Update(eps);CHKERRQ(ierr);
     }
   } else {
@@ -355,9 +409,14 @@ PetscErrorCode EPSSolve_Power(EPS eps)
   }
   if (power->nonlinear) {
     ierr = BVGetColumn(eps->V,0,&v);CHKERRQ(ierr);
+    if (eps->nini) {
+      /* We scale the initial vector back if the initial vector was provided by users */
+      ierr = VecScale(v,power->norm0);CHKERRQ(ierr);
+    }
     ierr = EPSPowerUpdateFunctionB(eps,v,Bx);CHKERRQ(ierr);
     ierr = VecNorm(Bx,NORM_2,&norm);CHKERRQ(ierr);
-    ierr = Normalize(Bx,norm,NULL);CHKERRQ(ierr);
+    ierr = FirstNonzeroIdx(Bx,&power->idx,&power->p);CHKERRQ(ierr);
+    ierr = Normalize(Bx,norm,power->idx,power->p,NULL);CHKERRQ(ierr);
     ierr = BVRestoreColumn(eps->V,0,&v);CHKERRQ(ierr);
   }
 
@@ -383,7 +442,7 @@ PetscErrorCode EPSSolve_Power(EPS eps)
     if (power->nonlinear) {
       ierr = EPSPowerUpdateFunctionB(eps,y,Bx);CHKERRQ(ierr);
       ierr = VecNorm(Bx,NORM_2,&norm);CHKERRQ(ierr);
-      ierr = Normalize(Bx,norm,&sign);CHKERRQ(ierr);
+      ierr = Normalize(Bx,norm,power->idx,power->p,&sign);CHKERRQ(ierr);
     } else {
       ierr = DSGetArray(eps->ds,DS_MAT_A,&T);CHKERRQ(ierr);
       ierr = BVSetActiveColumns(eps->V,0,k);CHKERRQ(ierr);
@@ -398,7 +457,8 @@ PetscErrorCode EPSSolve_Power(EPS eps)
       ierr = DSRestoreArray(eps->ds,DS_MAT_A,&T);CHKERRQ(ierr);
     }
 
-    if (power->nonlinear) theta = norm*sign;
+    /* Eigenvalue: 1/|Bx| */
+    if (power->nonlinear) theta = 1.0/norm*sign;
 
     if (power->shift_type == EPS_POWER_SHIFT_CONSTANT) { /* direct & inverse iteration */
 
@@ -411,7 +471,10 @@ PetscErrorCode EPSSolve_Power(EPS eps)
       ierr = VecAXPY(e,power->nonlinear?-1.0:-theta,v);CHKERRQ(ierr);
       ierr = BVRestoreColumn(eps->V,k,&v);CHKERRQ(ierr);
       ierr = VecNorm(e,NORM_2,&relerr);CHKERRQ(ierr);
-      relerr /= PetscAbsScalar(theta);
+      if (power->nonlinear)
+        relerr *= PetscAbsScalar(theta);
+      else
+        relerr /= PetscAbsScalar(theta);
 
     } else {  /* RQI */
 
@@ -430,11 +493,11 @@ PetscErrorCode EPSSolve_Power(EPS eps)
         rho = sigma;  /* if converged, restore original shift */
         ierr = STSetShift(eps->st,rho);CHKERRQ(ierr);
       } else {
-        rho = rho + theta/(delta*delta);  /* Rayleigh quotient R(v) */
+        rho = rho + PetscConj(theta)/(delta*delta);  /* Rayleigh quotient R(v) */
         if (power->shift_type == EPS_POWER_SHIFT_WILKINSON) {
           /* beta1 is the norm of the residual associated with R(v) */
           ierr = BVGetColumn(eps->V,k,&v);CHKERRQ(ierr);
-          ierr = VecAXPY(v,-theta/(delta*delta),y);CHKERRQ(ierr);
+          ierr = VecAXPY(v,-PetscConj(theta)/(delta*delta),y);CHKERRQ(ierr);
           ierr = BVRestoreColumn(eps->V,k,&v);CHKERRQ(ierr);
           ierr = BVScaleColumn(eps->V,k,1.0/delta);CHKERRQ(ierr);
           ierr = BVNormColumn(eps->V,k,NORM_2,&norm1);CHKERRQ(ierr);
@@ -461,10 +524,10 @@ PetscErrorCode EPSSolve_Power(EPS eps)
         ierr = KSPGetConvergedReason(ksp,&reason);CHKERRQ(ierr);
         if (reason) {
           ierr = PetscInfo(eps,"Factorization failed, repeat with a perturbed shift\n");CHKERRQ(ierr);
-          rho *= 1+PETSC_MACHINE_EPSILON;
+          rho *= 1+10*PETSC_MACHINE_EPSILON;
           ierr = STSetShift(eps->st,rho);CHKERRQ(ierr);
           ierr = KSPGetConvergedReason(ksp,&reason);CHKERRQ(ierr);
-          if (reason) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_CONV_FAILED,"Second factorization failed");
+          if (reason) SETERRQ(PetscObjectComm((PetscObject)ksp),PETSC_ERR_CONV_FAILED,"Second factorization failed");
         }
         ierr = KSPSetErrorIfNotConverged(ksp,PETSC_TRUE);CHKERRQ(ierr);
       }
@@ -472,7 +535,7 @@ PetscErrorCode EPSSolve_Power(EPS eps)
     eps->errest[eps->nconv] = relerr;
 
     /* normalize */
-    if (!power->nonlinear) { ierr = Normalize(y,norm,NULL);CHKERRQ(ierr); }
+    if (!power->nonlinear) { ierr = Normalize(y,norm,power->idx,power->p,NULL);CHKERRQ(ierr); }
     ierr = BVInsertVec(eps->V,k,y);CHKERRQ(ierr);
 
     if (power->update) {
@@ -501,9 +564,135 @@ PetscErrorCode EPSSolve_Power(EPS eps)
     ierr = DSSetState(eps->ds,DS_STATE_RAW);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
-#endif
 }
 
+PetscErrorCode EPSSolve_TS_Power(EPS eps)
+{
+  PetscErrorCode     ierr;
+  EPS_POWER          *power = (EPS_POWER*)eps->data;
+  PetscInt           k,ld;
+  Vec                v,w,y,e,z;
+  KSP                ksp;
+  PetscReal          relerr=1.0,relerrl,delta;
+  PetscScalar        theta,rho,alpha,sigma;
+  PetscBool          breakdown,breakdownl;
+  KSPConvergedReason reason;
+
+  PetscFunctionBegin;
+  e = eps->work[0];
+  v = eps->work[1];
+  w = eps->work[2];
+
+  if (power->shift_type != EPS_POWER_SHIFT_CONSTANT) { ierr = STGetKSP(eps->st,&ksp);CHKERRQ(ierr); }
+  ierr = DSGetLeadingDimension(eps->ds,&ld);CHKERRQ(ierr);
+  ierr = EPSGetStartVector(eps,0,NULL);CHKERRQ(ierr);
+  ierr = EPSGetLeftStartVector(eps,0,NULL);CHKERRQ(ierr);
+  ierr = BVBiorthonormalizeColumn(eps->V,eps->W,0,NULL);CHKERRQ(ierr);
+  ierr = BVCopyVec(eps->V,0,v);CHKERRQ(ierr);
+  ierr = BVCopyVec(eps->W,0,w);CHKERRQ(ierr);
+  ierr = STGetShift(eps->st,&sigma);CHKERRQ(ierr);    /* original shift */
+  rho = sigma;
+
+  while (eps->reason == EPS_CONVERGED_ITERATING) {
+    eps->its++;
+    k = eps->nconv;
+
+    /* y = OP v, z = OP' w */
+    ierr = BVGetColumn(eps->V,k,&y);CHKERRQ(ierr);
+    ierr = STApply(eps->st,v,y);CHKERRQ(ierr);
+    ierr = BVRestoreColumn(eps->V,k,&y);CHKERRQ(ierr);
+    ierr = BVGetColumn(eps->W,k,&z);CHKERRQ(ierr);
+    ierr = STApplyHermitianTranspose(eps->st,w,z);CHKERRQ(ierr);
+    ierr = BVRestoreColumn(eps->W,k,&z);CHKERRQ(ierr);
+
+    /* purge previously converged eigenvectors */
+    ierr = BVBiorthogonalizeColumn(eps->V,eps->W,k);CHKERRQ(ierr);
+
+    /* theta = (w,y)_B */
+    ierr = BVSetActiveColumns(eps->V,k,k+1);CHKERRQ(ierr);
+    ierr = BVDotVec(eps->V,w,&theta);CHKERRQ(ierr);
+    theta = PetscConj(theta);
+
+    if (power->shift_type == EPS_POWER_SHIFT_CONSTANT) { /* direct & inverse iteration */
+
+      /* approximate eigenvalue is the Rayleigh quotient */
+      eps->eigr[eps->nconv] = theta;
+
+      /* compute relative errors as ||y-theta v||_2/|theta| and ||z-conj(theta) w||_2/|theta|*/
+      ierr = BVCopyVec(eps->V,k,e);CHKERRQ(ierr);
+      ierr = VecAXPY(e,-theta,v);CHKERRQ(ierr);
+      ierr = VecNorm(e,NORM_2,&relerr);CHKERRQ(ierr);
+      ierr = BVCopyVec(eps->W,k,e);CHKERRQ(ierr);
+      ierr = VecAXPY(e,-PetscConj(theta),w);CHKERRQ(ierr);
+      ierr = VecNorm(e,NORM_2,&relerrl);CHKERRQ(ierr);
+      relerr = PetscMax(relerr,relerrl)/PetscAbsScalar(theta);
+    }
+
+    /* normalize */
+    ierr = BVSetActiveColumns(eps->V,k,k+1);CHKERRQ(ierr);
+    ierr = BVGetColumn(eps->W,k,&z);CHKERRQ(ierr);
+    ierr = BVDotVec(eps->V,z,&alpha);CHKERRQ(ierr);
+    ierr = BVRestoreColumn(eps->W,k,&z);CHKERRQ(ierr);
+    delta = PetscSqrtReal(PetscAbsScalar(alpha));
+    if (delta==0.0) SETERRQ(PetscObjectComm((PetscObject)eps),1,"Breakdown in two-sided Power/RQI");
+    ierr = BVScaleColumn(eps->V,k,1.0/PetscConj(alpha/delta));CHKERRQ(ierr);
+    ierr = BVScaleColumn(eps->W,k,1.0/delta);CHKERRQ(ierr);
+    ierr = BVCopyVec(eps->V,k,v);CHKERRQ(ierr);
+    ierr = BVCopyVec(eps->W,k,w);CHKERRQ(ierr);
+
+    if (power->shift_type == EPS_POWER_SHIFT_RAYLEIGH) { /* RQI */
+
+      /* compute relative error */
+      if (rho == 0.0) relerr = PETSC_MAX_REAL;
+      else relerr = 1.0 / PetscAbsScalar(delta*rho);
+
+      /* approximate eigenvalue is the shift */
+      eps->eigr[eps->nconv] = rho;
+
+      /* compute new shift */
+      if (relerr<eps->tol) {
+        rho = sigma;  /* if converged, restore original shift */
+        ierr = STSetShift(eps->st,rho);CHKERRQ(ierr);
+      } else {
+        rho = rho + PetscConj(theta)/(delta*delta);  /* Rayleigh quotient R(v) */
+        /* update operator according to new shift */
+        ierr = KSPSetErrorIfNotConverged(ksp,PETSC_FALSE);CHKERRQ(ierr);
+        ierr = STSetShift(eps->st,rho);CHKERRQ(ierr);
+        ierr = KSPGetConvergedReason(ksp,&reason);CHKERRQ(ierr);
+        if (reason) {
+          ierr = PetscInfo(eps,"Factorization failed, repeat with a perturbed shift\n");CHKERRQ(ierr);
+          rho *= 1+10*PETSC_MACHINE_EPSILON;
+          ierr = STSetShift(eps->st,rho);CHKERRQ(ierr);
+          ierr = KSPGetConvergedReason(ksp,&reason);CHKERRQ(ierr);
+          if (reason) SETERRQ(PetscObjectComm((PetscObject)ksp),PETSC_ERR_CONV_FAILED,"Second factorization failed");
+        }
+        ierr = KSPSetErrorIfNotConverged(ksp,PETSC_TRUE);CHKERRQ(ierr);
+      }
+    }
+    eps->errest[eps->nconv] = relerr;
+
+    /* if relerr<tol, accept eigenpair */
+    if (relerr<eps->tol) {
+      eps->nconv = eps->nconv + 1;
+      if (eps->nconv<eps->nev) {
+        ierr = EPSGetStartVector(eps,eps->nconv,&breakdown);CHKERRQ(ierr);
+        ierr = EPSGetLeftStartVector(eps,eps->nconv,&breakdownl);CHKERRQ(ierr);
+        if (breakdown || breakdownl) {
+          eps->reason = EPS_DIVERGED_BREAKDOWN;
+          ierr = PetscInfo(eps,"Unable to generate more start vectors\n");CHKERRQ(ierr);
+          break;
+        }
+        ierr = BVBiorthonormalizeColumn(eps->V,eps->W,eps->nconv,NULL);CHKERRQ(ierr);
+      }
+    }
+    ierr = EPSMonitor(eps,eps->its,eps->nconv,eps->eigr,eps->eigi,eps->errest,PetscMin(eps->nconv+1,eps->nev));CHKERRQ(ierr);
+    ierr = (*eps->stopping)(eps,eps->its,eps->max_it,eps->nconv,eps->nev,&eps->reason,eps->stoppingctx);CHKERRQ(ierr);
+  }
+
+  ierr = DSSetDimensions(eps->ds,eps->nconv,0,0,0);CHKERRQ(ierr);
+  ierr = DSSetState(eps->ds,DS_STATE_RAW);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
 
 PetscErrorCode EPSStopping_Power(EPS eps,PetscInt its,PetscInt max_it,PetscInt nconv,PetscInt nev,EPSConvergedReason *reason,void *ctx)
 {
@@ -584,7 +773,7 @@ static PetscErrorCode EPSPowerSetShiftType_Power(EPS eps,EPSPowerShiftType shift
    iteration. This can be used to emulate the Rayleigh Quotient Iteration
    (RQI) method.
 
-   Logically Collective on EPS
+   Logically Collective on eps
 
    Input Parameters:
 +  eps - the eigenproblem solver context
@@ -670,7 +859,7 @@ static PetscErrorCode EPSPowerSetNonlinear_Power(EPS eps,PetscBool nonlinear)
 /*@
    EPSPowerSetNonlinear - Sets a flag to indicate that the problem is nonlinear.
 
-   Logically Collective on EPS
+   Logically Collective on eps
 
    Input Parameters:
 +  eps - the eigenproblem solver context
@@ -738,7 +927,7 @@ PetscErrorCode EPSPowerGetNonlinear(EPS eps,PetscBool *nonlinear)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
-  PetscValidPointer(nonlinear,2);
+  PetscValidBoolPointer(nonlinear,2);
   ierr = PetscUseMethod(eps,"EPSPowerGetNonlinear_C",(EPS,PetscBool*),(eps,nonlinear));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -748,7 +937,7 @@ static PetscErrorCode EPSPowerSetUpdate_Power(EPS eps,PetscBool update)
   EPS_POWER *power = (EPS_POWER*)eps->data;
 
   PetscFunctionBegin;
-  if (!power->nonlinear) SETERRQ(PETSC_COMM_WORLD,PETSC_ERR_ARG_INCOMP,"This option does not make sense for linear problems \n");
+  if (!power->nonlinear) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_INCOMP,"This option does not make sense for linear problems");
   power->update = update;
   eps->state = EPS_STATE_INITIAL;
   PetscFunctionReturn(0);
@@ -758,7 +947,7 @@ static PetscErrorCode EPSPowerSetUpdate_Power(EPS eps,PetscBool update)
    EPSPowerSetUpdate - Sets a flag to indicate that the residual is updated monolithically
    for nonlinear problems. This potentially has a better convergence.
 
-   Logically Collective on EPS
+   Logically Collective on eps
 
    Input Parameters:
 +  eps - the eigenproblem solver context
@@ -813,7 +1002,7 @@ PetscErrorCode EPSPowerGetUpdate(EPS eps,PetscBool *update)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
-  PetscValidPointer(update,2);
+  PetscValidBoolPointer(update,2);
   ierr = PetscUseMethod(eps,"EPSPowerGetUpdate_C",(EPS,PetscBool*),(eps,update));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -836,7 +1025,7 @@ static PetscErrorCode EPSPowerSetSNES_Power(EPS eps,SNES snes)
    EPSPowerSetSNES - Associate a nonlinear solver object (SNES) to the
    eigenvalue solver (to be used in nonlinear inverse iteration).
 
-   Collective on EPS
+   Collective on eps
 
    Input Parameters:
 +  eps  - the eigenvalue solver
@@ -870,6 +1059,7 @@ static PetscErrorCode EPSPowerGetSNES_Power(EPS eps,SNES *snes)
     ierr = SNESSetOptionsPrefix(power->snes,((PetscObject)eps)->prefix);CHKERRQ(ierr);
     ierr = SNESAppendOptionsPrefix(power->snes,"eps_power_");CHKERRQ(ierr);
     ierr = PetscLogObjectParent((PetscObject)eps,(PetscObject)power->snes);CHKERRQ(ierr);
+    ierr = PetscObjectSetOptions((PetscObject)power->snes,((PetscObject)eps)->options);CHKERRQ(ierr);
   }
   *snes = power->snes;
   PetscFunctionReturn(0);
@@ -948,7 +1138,9 @@ PetscErrorCode EPSView_Power(EPS eps,PetscViewer viewer)
         ierr = PetscViewerASCIIPrintf(viewer,"  updating the residual monolithically\n");CHKERRQ(ierr);
       }
       if (!power->snes) { ierr = EPSPowerGetSNES(eps,&power->snes);CHKERRQ(ierr); }
+      ierr = PetscViewerASCIIPushTab(viewer);CHKERRQ(ierr);
       ierr = SNESView(power->snes,viewer);CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPopTab(viewer);CHKERRQ(ierr);
     } else {
       ierr = PetscViewerASCIIPrintf(viewer,"  %s shifts\n",EPSPowerShiftTypes[power->shift_type]);CHKERRQ(ierr);
     }
@@ -960,9 +1152,20 @@ PetscErrorCode EPSComputeVectors_Power(EPS eps)
 {
   PetscErrorCode ierr;
   EPS_POWER      *power = (EPS_POWER*)eps->data;
+  PetscReal      norm;
+  PetscInt       i;
 
   PetscFunctionBegin;
-  if (!power->nonlinear) {
+  if (eps->twosided) {
+    ierr = EPSComputeVectors_Twosided(eps);CHKERRQ(ierr);
+    /* normalize (no need to take care of 2x2 blocks */
+    for (i=0;i<eps->nconv;i++) {
+      ierr = BVNormColumn(eps->V,i,NORM_2,&norm);CHKERRQ(ierr);
+      ierr = BVScaleColumn(eps->V,i,1.0/norm);CHKERRQ(ierr);
+      ierr = BVNormColumn(eps->W,i,NORM_2,&norm);CHKERRQ(ierr);
+      ierr = BVScaleColumn(eps->W,i,1.0/norm);CHKERRQ(ierr);
+    }
+  } else if (!power->nonlinear) {
     ierr = EPSComputeVectors_Schur(eps);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
@@ -977,14 +1180,19 @@ PetscErrorCode EPSSetDefaultST_Power(EPS eps)
 
   PetscFunctionBegin;
   if (power->nonlinear) {
+    eps->categ=EPS_CATEGORY_PRECOND;
     ierr = STGetKSP(eps->st,&ksp);CHKERRQ(ierr);
+    /* Set ST as STPRECOND so it can carry one preconditioning matrix
+     * It is useful when A and B are shell matrices
+     */
+    ierr = STSetType(eps->st,STPRECOND);CHKERRQ(ierr);
     ierr = KSPGetPC(ksp,&pc);CHKERRQ(ierr);
     ierr = PCSetType(pc,PCNONE);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
-PETSC_EXTERN PetscErrorCode EPSCreate_Power(EPS eps)
+SLEPC_EXTERN PetscErrorCode EPSCreate_Power(EPS eps)
 {
   EPS_POWER      *ctx;
   PetscErrorCode ierr;
@@ -994,9 +1202,9 @@ PETSC_EXTERN PetscErrorCode EPSCreate_Power(EPS eps)
   eps->data = (void*)ctx;
 
   eps->useds = PETSC_TRUE;
+  eps->hasts = PETSC_TRUE;
   eps->categ = EPS_CATEGORY_OTHER;
 
-  eps->ops->solve          = EPSSolve_Power;
   eps->ops->setup          = EPSSetUp_Power;
   eps->ops->setfromoptions = EPSSetFromOptions_Power;
   eps->ops->reset          = EPSReset_Power;
@@ -1017,4 +1225,3 @@ PETSC_EXTERN PetscErrorCode EPSCreate_Power(EPS eps)
   ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSPowerGetSNES_C",EPSPowerGetSNES_Power);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
-
