@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -20,7 +20,7 @@
    References:
 
        [1] "Subspace Iteration in SLEPc", SLEPc Technical Report STR-3,
-           available at http://slepc.upv.es.
+           available at https://slepc.upv.es.
 */
 
 #include <slepc/private/epsimpl.h>
@@ -274,7 +274,7 @@ PetscErrorCode EPSSolve_Subspace(EPS eps)
 
   ierr = PetscFree3(rsd,itrsd,itrsdold);CHKERRQ(ierr);
   ierr = BVDestroy(&AV);CHKERRQ(ierr);
-  ierr = MatDestroy(&S);CHKERRQ(ierr);
+  ierr = STRestoreOperator(eps->st,&S);CHKERRQ(ierr);
   /* truncate Schur decomposition and change the state to raw so that
      DSVectors() computes eigenvectors from scratch */
   ierr = DSSetDimensions(eps->ds,eps->nconv,0,0,0);CHKERRQ(ierr);
@@ -291,7 +291,7 @@ PetscErrorCode EPSDestroy_Subspace(EPS eps)
   PetscFunctionReturn(0);
 }
 
-PETSC_EXTERN PetscErrorCode EPSCreate_Subspace(EPS eps)
+SLEPC_EXTERN PetscErrorCode EPSCreate_Subspace(EPS eps)
 {
   PetscFunctionBegin;
   eps->useds = PETSC_TRUE;
