@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -75,8 +75,6 @@ PetscErrorCode EPSComputeVectors_Indefinite(EPS eps)
   Vec            v;
 #if !defined(PETSC_USE_COMPLEX)
   Vec            v1;
-  PetscScalar    tmp;
-  PetscReal      norm,normi;
 #endif
 
   PetscFunctionBegin;
@@ -96,11 +94,7 @@ PetscErrorCode EPSComputeVectors_Indefinite(EPS eps)
     if (eps->eigi[i] != 0.0) {
       ierr = BVGetColumn(eps->V,i,&v);CHKERRQ(ierr);
       ierr = BVGetColumn(eps->V,i+1,&v1);CHKERRQ(ierr);
-      ierr = VecNorm(v,NORM_2,&norm);CHKERRQ(ierr);
-      ierr = VecNorm(v1,NORM_2,&normi);CHKERRQ(ierr);
-      tmp = 1.0 / SlepcAbsEigenvalue(norm,normi);
-      ierr = VecScale(v,tmp);CHKERRQ(ierr);
-      ierr = VecScale(v1,tmp);CHKERRQ(ierr);
+      ierr = VecNormalizeComplex(v,v1,PETSC_TRUE,NULL);CHKERRQ(ierr);
       ierr = BVRestoreColumn(eps->V,i,&v);CHKERRQ(ierr);
       ierr = BVRestoreColumn(eps->V,i+1,&v1);CHKERRQ(ierr);
       i++;
@@ -115,6 +109,30 @@ PetscErrorCode EPSComputeVectors_Indefinite(EPS eps)
   PetscFunctionReturn(0);
 }
 
+/*
+  EPSComputeVectors_Twosided - Adjust left eigenvectors in generalized problems: y = B^-* y.
+ */
+PetscErrorCode EPSComputeVectors_Twosided(EPS eps)
+{
+  PetscErrorCode ierr;
+  PetscInt       i;
+  Vec            w,y;
+
+  PetscFunctionBegin;
+  if (!eps->twosided || !eps->isgeneralized) PetscFunctionReturn(0);
+  ierr = EPSSetWorkVecs(eps,1);CHKERRQ(ierr);
+  w = eps->work[0];
+  for (i=0;i<eps->nconv;i++) {
+    ierr = BVCopyVec(eps->W,i,w);CHKERRQ(ierr);
+    ierr = VecConjugate(w);CHKERRQ(ierr);
+    ierr = BVGetColumn(eps->W,i,&y);CHKERRQ(ierr);
+    ierr = STMatSolveTranspose(eps->st,w,y);CHKERRQ(ierr);
+    ierr = VecConjugate(y);CHKERRQ(ierr);
+    ierr = BVRestoreColumn(eps->W,i,&y);CHKERRQ(ierr);
+  }
+  PetscFunctionReturn(0);
+}
+
 /*
   EPSComputeVectors_Schur - Compute eigenvectors from the vectors
   provided by the eigensolver. This version is intended for solvers
@@ -131,8 +149,6 @@ PetscErrorCode EPSComputeVectors_Schur(EPS eps)
   Vec            z,v;
 #if !defined(PETSC_USE_COMPLEX)
   Vec            v1;
-  PetscScalar    tmp;
-  PetscReal      norm,normi;
 #endif
 
   PetscFunctionBegin;
@@ -174,11 +190,7 @@ PetscErrorCode EPSComputeVectors_Schur(EPS eps)
       if (eps->eigi[i] != 0.0) {
         ierr = BVGetColumn(eps->V,i,&v);CHKERRQ(ierr);
         ierr = BVGetColumn(eps->V,i+1,&v1);CHKERRQ(ierr);
-        ierr = VecNorm(v,NORM_2,&norm);CHKERRQ(ierr);
-        ierr = VecNorm(v1,NORM_2,&normi);CHKERRQ(ierr);
-        tmp = 1.0 / SlepcAbsEigenvalue(norm,normi);
-        ierr = VecScale(v,tmp);CHKERRQ(ierr);
-        ierr = VecScale(v1,tmp);CHKERRQ(ierr);
+        ierr = VecNormalizeComplex(v,v1,PETSC_TRUE,NULL);CHKERRQ(ierr);
         ierr = BVRestoreColumn(eps->V,i,&v);CHKERRQ(ierr);
         ierr = BVRestoreColumn(eps->V,i+1,&v1);CHKERRQ(ierr);
         i++;
@@ -191,20 +203,65 @@ PetscErrorCode EPSComputeVectors_Schur(EPS eps)
       }
     }
   }
+
+  /* left eigenvectors from eps->dsts */
+  if (eps->twosided) {
+    ierr = DSVectors(eps->dsts,DS_MAT_X,NULL,NULL);CHKERRQ(ierr);
+    /* W = W * Z */
+    ierr = DSGetMat(eps->dsts,DS_MAT_X,&Z);CHKERRQ(ierr);
+    ierr = BVSetActiveColumns(eps->W,0,n);CHKERRQ(ierr);
+    ierr = BVMultInPlace(eps->W,Z,0,n);CHKERRQ(ierr);
+    ierr = MatDestroy(&Z);CHKERRQ(ierr);
+    /* Fix left eigenvectors if balancing was used */
+    if (eps->balance!=EPS_BALANCE_NONE && eps->D) {
+      for (i=0;i<n;i++) {
+        ierr = BVGetColumn(eps->W,i,&z);CHKERRQ(ierr);
+        ierr = VecPointwiseMult(z,z,eps->D);CHKERRQ(ierr);
+        ierr = BVRestoreColumn(eps->W,i,&z);CHKERRQ(ierr);
+      }
+    }
+    ierr = EPSComputeVectors_Twosided(eps);CHKERRQ(ierr);
+    /* normalize */
+    for (i=0;i<eps->nconv;i++) {
+#if !defined(PETSC_USE_COMPLEX)
+      if (eps->eigi[i]!=0.0) {   /* first eigenvalue of a complex conjugate pair */
+        ierr = BVGetColumn(eps->W,i,&v);CHKERRQ(ierr);
+        ierr = BVGetColumn(eps->W,i+1,&v1);CHKERRQ(ierr);
+        ierr = VecNormalizeComplex(v,v1,PETSC_TRUE,NULL);CHKERRQ(ierr);
+        ierr = BVRestoreColumn(eps->W,i,&v);CHKERRQ(ierr);
+        ierr = BVRestoreColumn(eps->W,i+1,&v1);CHKERRQ(ierr);
+        i++;
+      } else   /* real eigenvalue */
+#endif
+      {
+        ierr = BVGetColumn(eps->W,i,&v);CHKERRQ(ierr);
+        ierr = VecNormalize(v,NULL);CHKERRQ(ierr);
+        ierr = BVRestoreColumn(eps->W,i,&v);CHKERRQ(ierr);
+      }
+    }
+#if !defined(PETSC_USE_COMPLEX)
+    for (i=0;i<eps->nconv-1;i++) {
+      if (eps->eigi[i] != 0.0) {
+        if (eps->eigi[i] > 0.0) { ierr = BVScaleColumn(eps->W,i+1,-1.0);CHKERRQ(ierr); }
+        i++;
+      }
+    }
+#endif
+  }
   PetscFunctionReturn(0);
 }
 
 /*@
    EPSSetWorkVecs - Sets a number of work vectors into an EPS object.
 
-   Collective on EPS
+   Collective on eps
 
    Input Parameters:
 +  eps - eigensolver context
 -  nw  - number of work vectors to allocate
 
    Developers Note:
-   This is PETSC_EXTERN because it may be required by user plugin EPS
+   This is SLEPC_EXTERN because it may be required by user plugin EPS
    implementations.
 
    Level: developer
@@ -286,7 +343,7 @@ PetscErrorCode EPSConvergedNorm(EPS eps,PetscScalar eigr,PetscScalar eigi,PetscR
    EPSStoppingBasic - Default routine to determine whether the outer eigensolver
    iteration must be stopped.
 
-   Collective on EPS
+   Collective on eps
 
    Input Parameters:
 +  eps    - eigensolver context obtained from EPSCreate()
@@ -346,8 +403,6 @@ PetscErrorCode EPSComputeRitzVector(EPS eps,PetscScalar *Zr,PetscScalar *Zi,BV V
   PetscReal      norm;
 #if !defined(PETSC_USE_COMPLEX)
   Vec            z;
-  PetscReal      normi;
-  PetscScalar    tmp;
 #endif
 
   PetscFunctionBegin;
@@ -394,11 +449,7 @@ PetscErrorCode EPSComputeRitzVector(EPS eps,PetscScalar *Zr,PetscScalar *Zi,BV V
   if (eps->balance!=EPS_BALANCE_NONE && eps->D) {
 #if !defined(PETSC_USE_COMPLEX)
     if (Zi) {
-      ierr = VecNorm(x,NORM_2,&norm);CHKERRQ(ierr);
-      ierr = VecNorm(y,NORM_2,&normi);CHKERRQ(ierr);
-      tmp = 1.0 / SlepcAbsEigenvalue(norm,normi);
-      ierr = VecScale(x,tmp);CHKERRQ(ierr);
-      ierr = VecScale(y,tmp);CHKERRQ(ierr);
+      ierr = VecNormalizeComplex(x,y,PETSC_TRUE,NULL);CHKERRQ(ierr);
     } else
 #endif
     {
@@ -454,7 +505,7 @@ PetscErrorCode EPSBuildBalance_Krylov(EPS eps)
       }
       /* Compute r=D\(A'Dz) */
       ierr = VecPointwiseMult(z,z,eps->D);CHKERRQ(ierr);
-      ierr = STApplyTranspose(eps->st,z,r);CHKERRQ(ierr);
+      ierr = STApplyHermitianTranspose(eps->st,z,r);CHKERRQ(ierr);
       ierr = VecPointwiseDivide(r,r,eps->D);CHKERRQ(ierr);
     }
 
