@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2018, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -43,7 +43,6 @@ PetscErrorCode EPSSetDefaultST_Precond(EPS eps)
   PetscFunctionBegin;
   if (!((PetscObject)eps->st)->type_name) {
     ierr = STSetType(eps->st,STPRECOND);CHKERRQ(ierr);
-    ierr = STPrecondSetKSPHasMat(eps->st,PETSC_TRUE);CHKERRQ(ierr);
   }
   ierr = STGetKSP(eps->st,&ksp);CHKERRQ(ierr);
   if (!((PetscObject)ksp)->type_name) {
@@ -54,7 +53,7 @@ PetscErrorCode EPSSetDefaultST_Precond(EPS eps)
 
 /*
    This is done by preconditioned eigensolvers that can also use the KSP.
-   It sets STPRECOND with the default KSP (GMRES).
+   It sets STPRECOND with the default KSP (GMRES) and maxit=5.
 */
 PetscErrorCode EPSSetDefaultST_GMRES(EPS eps)
 {
@@ -64,10 +63,54 @@ PetscErrorCode EPSSetDefaultST_GMRES(EPS eps)
   PetscFunctionBegin;
   if (!((PetscObject)eps->st)->type_name) {
     ierr = STSetType(eps->st,STPRECOND);CHKERRQ(ierr);
-    ierr = STPrecondSetKSPHasMat(eps->st,PETSC_TRUE);CHKERRQ(ierr);
   }
+  ierr = STPrecondSetKSPHasMat(eps->st,PETSC_TRUE);CHKERRQ(ierr);
   ierr = STGetKSP(eps->st,&ksp);CHKERRQ(ierr);
-  ierr = KSPSetTolerances(ksp,PETSC_DEFAULT,PETSC_DEFAULT,PETSC_DEFAULT,5);CHKERRQ(ierr);
+  if (!((PetscObject)ksp)->type_name) {
+    ierr = KSPSetType(ksp,KSPGMRES);CHKERRQ(ierr);
+    ierr = KSPSetTolerances(ksp,PETSC_DEFAULT,PETSC_DEFAULT,PETSC_DEFAULT,5);CHKERRQ(ierr);
+  }
+  PetscFunctionReturn(0);
+}
+
+/*
+   Check that the ST selected by the user is compatible with the EPS solver and options
+*/
+PetscErrorCode EPSCheckCompatibleST(EPS eps)
+{
+  PetscErrorCode ierr;
+  PetscBool      precond,shift,sinvert,cayley,lyapii;
+#if defined(PETSC_USE_COMPLEX)
+  PetscScalar    sigma;
+#endif
+
+  PetscFunctionBegin;
+  ierr = PetscObjectTypeCompare((PetscObject)eps->st,STPRECOND,&precond);CHKERRQ(ierr);
+  ierr = PetscObjectTypeCompare((PetscObject)eps->st,STSHIFT,&shift);CHKERRQ(ierr);
+  ierr = PetscObjectTypeCompare((PetscObject)eps->st,STSINVERT,&sinvert);CHKERRQ(ierr);
+  ierr = PetscObjectTypeCompare((PetscObject)eps->st,STCAYLEY,&cayley);CHKERRQ(ierr);
+
+  /* preconditioned eigensolvers */
+  if (eps->categ==EPS_CATEGORY_PRECOND && !precond) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"This solver requires ST=PRECOND");
+  if (eps->categ!=EPS_CATEGORY_PRECOND && precond) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"STPRECOND is intended for preconditioned eigensolvers only");
+
+  /* harmonic extraction */
+  if (!(precond || shift) && eps->extraction && eps->extraction!=EPS_RITZ) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Cannot use a spectral transformation combined with harmonic extraction");
+
+  /* real shifts in Hermitian problems */
+#if defined(PETSC_USE_COMPLEX)
+  ierr = STGetShift(eps->st,&sigma);CHKERRQ(ierr);
+  if (eps->ishermitian && PetscImaginaryPart(sigma) != 0.0) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Hermitian problems are not compatible with complex shifts");
+#endif
+
+  /* Cayley with PGNHEP */
+  if (cayley && eps->problem_type == EPS_PGNHEP) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Cayley spectral transformation is not compatible with PGNHEP");
+
+  /* make sure that the user does not specify smallest magnitude with shift-and-invert */
+  if ((cayley || sinvert) && (eps->categ==EPS_CATEGORY_KRYLOV || eps->categ==EPS_CATEGORY_OTHER)) {
+    ierr = PetscObjectTypeCompare((PetscObject)eps,EPSLYAPII,&lyapii);CHKERRQ(ierr);
+    if (!lyapii && eps->which!=EPS_TARGET_MAGNITUDE && eps->which!=EPS_TARGET_REAL && eps->which!=EPS_TARGET_IMAGINARY && eps->which!=EPS_ALL && eps->which!=EPS_WHICH_USER) SETERRQ(PetscObjectComm((PetscObject)eps),1,"Shift-and-invert requires a target 'which' (see EPSSetWhichEigenpairs), for instance -st_type sinvert -eps_target 0 -eps_target_magnitude");
+  }
   PetscFunctionReturn(0);
 }
 
@@ -76,7 +119,7 @@ PetscErrorCode EPSSetDefaultST_GMRES(EPS eps)
    execution of the eigensolver. Then calls STSetUp() for any set-up
    operations associated to the ST object.
 
-   Collective on EPS
+   Collective on eps
 
    Input Parameter:
 .  eps   - eigenproblem solver context
@@ -96,10 +139,7 @@ PetscErrorCode EPSSetUp(EPS eps)
   Mat            A,B;
   SlepcSC        sc;
   PetscInt       k,nmat;
-  PetscBool      flg,istrivial,precond;
-#if defined(PETSC_USE_COMPLEX)
-  PetscScalar    sigma;
-#endif
+  PetscBool      flg,istrivial;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
@@ -116,12 +156,13 @@ PetscErrorCode EPSSetUp(EPS eps)
   if (!eps->st) { ierr = EPSGetST(eps,&eps->st);CHKERRQ(ierr); }
   ierr = EPSSetDefaultST(eps);CHKERRQ(ierr);
 
-  ierr = PetscObjectTypeCompare((PetscObject)eps->st,STPRECOND,&precond);CHKERRQ(ierr);
-  if (eps->categ==EPS_CATEGORY_PRECOND && !precond) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"This solver requires ST=PRECOND");
-  if (eps->categ!=EPS_CATEGORY_PRECOND && precond) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"STPRECOND is intended for preconditioned eigensolvers only");
-
   ierr = STSetTransform(eps->st,PETSC_TRUE);CHKERRQ(ierr);
   if (eps->useds && !eps->ds) { ierr = EPSGetDS(eps,&eps->ds);CHKERRQ(ierr); }
+  if (eps->twosided) {
+    if (!eps->hasts) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"This solver does not support computing left eigenvectors (no two-sided variant)");
+    if (eps->ishermitian) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Two-sided methods are not intended for symmetric problems");
+    if (!eps->dsts) { ierr = DSDuplicate(eps->ds,&eps->dsts);CHKERRQ(ierr); }
+  }
   if (!eps->rg) { ierr = EPSGetRG(eps,&eps->rg);CHKERRQ(ierr); }
   if (!((PetscObject)eps->rg)->type_name) {
     ierr = RGSetType(eps->rg,RGINTERVAL);CHKERRQ(ierr);
@@ -177,10 +218,6 @@ PetscErrorCode EPSSetUp(EPS eps)
     }
   }
 
-  /* check extraction */
-  ierr = PetscObjectTypeCompareAny((PetscObject)eps->st,&flg,STPRECOND,STSHIFT,"");CHKERRQ(ierr);
-  if (!flg && eps->extraction && eps->extraction!=EPS_RITZ) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Cannot use a spectral transformation combined with harmonic extraction");
-
   /* set tolerance if not yet set */
   if (eps->tol==PETSC_DEFAULT) eps->tol = SLEPC_DEFAULT_TOL;
 
@@ -234,42 +271,36 @@ PetscErrorCode EPSSetUp(EPS eps)
   eps->sc->map    = NULL;
   eps->sc->mapobj = NULL;
 
-  if (eps->useds) {
-    /* fill sorting criterion for DS */
+  /* fill sorting criterion for DS */
+  if (eps->useds && eps->which!=EPS_ALL) {
     ierr = DSGetSlepcSC(eps->ds,&sc);CHKERRQ(ierr);
     ierr = RGIsTrivial(eps->rg,&istrivial);CHKERRQ(ierr);
-    if (eps->which!=EPS_ALL) {
-      sc->rg            = istrivial? NULL: eps->rg;
-      sc->comparison    = eps->sc->comparison;
-      sc->comparisonctx = eps->sc->comparisonctx;
-      sc->map           = SlepcMap_ST;
-      sc->mapobj        = (PetscObject)eps->st;
+    sc->rg            = istrivial? NULL: eps->rg;
+    sc->comparison    = eps->sc->comparison;
+    sc->comparisonctx = eps->sc->comparisonctx;
+    sc->map           = SlepcMap_ST;
+    sc->mapobj        = (PetscObject)eps->st;
+    if (eps->twosided) {
+      ierr = DSSetSlepcSC(eps->dsts,sc);CHKERRQ(ierr);
     }
   }
 
   /* Build balancing matrix if required */
-  if (!eps->ishermitian && (eps->balance==EPS_BALANCE_ONESIDE || eps->balance==EPS_BALANCE_TWOSIDE)) {
-    if (!eps->D) {
-      ierr = BVCreateVec(eps->V,&eps->D);CHKERRQ(ierr);
-      ierr = PetscLogObjectParent((PetscObject)eps,(PetscObject)eps->D);CHKERRQ(ierr);
-    } else {
-      ierr = VecSet(eps->D,1.0);CHKERRQ(ierr);
+  if (eps->balance!=EPS_BALANCE_USER) {
+    ierr = STSetBalanceMatrix(eps->st,NULL);CHKERRQ(ierr);
+    if (!eps->ishermitian && (eps->balance==EPS_BALANCE_ONESIDE || eps->balance==EPS_BALANCE_TWOSIDE)) {
+      if (!eps->D) {
+        ierr = BVCreateVec(eps->V,&eps->D);CHKERRQ(ierr);
+        ierr = PetscLogObjectParent((PetscObject)eps,(PetscObject)eps->D);CHKERRQ(ierr);
+      }
+      ierr = EPSBuildBalance_Krylov(eps);CHKERRQ(ierr);
+      ierr = STSetBalanceMatrix(eps->st,eps->D);CHKERRQ(ierr);
     }
-    ierr = EPSBuildBalance_Krylov(eps);CHKERRQ(ierr);
-    ierr = STSetBalanceMatrix(eps->st,eps->D);CHKERRQ(ierr);
   }
 
   /* Setup ST */
   ierr = STSetUp(eps->st);CHKERRQ(ierr);
-
-#if defined(PETSC_USE_COMPLEX)
-  ierr = STGetShift(eps->st,&sigma);CHKERRQ(ierr);
-  if (eps->ishermitian && PetscImaginaryPart(sigma) != 0.0) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Hermitian problems are not compatible with complex shifts");
-#endif
-  ierr = PetscObjectTypeCompare((PetscObject)eps->st,STCAYLEY,&flg);CHKERRQ(ierr);
-  if (flg && eps->problem_type == EPS_PGNHEP) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Cayley spectral transformation is not compatible with PGNHEP");
-  ierr = PetscObjectTypeCompareAny((PetscObject)eps->st,&flg,STSINVERT,STCAYLEY,"");CHKERRQ(ierr);
-  if (flg && (eps->categ==EPS_CATEGORY_KRYLOV || eps->categ==EPS_CATEGORY_OTHER) && (eps->which!=EPS_TARGET_MAGNITUDE && eps->which!=EPS_TARGET_REAL && eps->which!=EPS_TARGET_IMAGINARY && eps->which!=EPS_ALL && eps->which!=EPS_WHICH_USER)) SETERRQ(PetscObjectComm((PetscObject)eps),1,"Shift-and-invert requires a target 'which' (see EPSSetWhichEigenpairs), for instance -st_type sinvert -eps_target 0 -eps_target_magnitude");
+  ierr = EPSCheckCompatibleST(eps);CHKERRQ(ierr);
 
   /* process deflation and initial vectors */
   if (eps->nds<0) {
@@ -286,16 +317,23 @@ PetscErrorCode EPSSetUp(EPS eps)
     ierr = SlepcBasisDestroy_Private(&eps->nini,&eps->IS);CHKERRQ(ierr);
     eps->nini = k;
   }
+  if (eps->twosided && eps->ninil<0) {
+    k = -eps->ninil;
+    if (k>eps->ncv) SETERRQ(PetscObjectComm((PetscObject)eps),1,"The number of left initial vectors is larger than ncv");
+    ierr = BVInsertVecs(eps->W,0,&k,eps->ISL,PETSC_TRUE);CHKERRQ(ierr);
+    ierr = SlepcBasisDestroy_Private(&eps->ninil,&eps->ISL);CHKERRQ(ierr);
+    eps->ninil = k;
+  }
 
   ierr = PetscLogEventEnd(EPS_SetUp,eps,0,0,0);CHKERRQ(ierr);
   eps->state = EPS_STATE_SETUP;
   PetscFunctionReturn(0);
 }
 
-/*@C
+/*@
    EPSSetOperators - Sets the matrices associated with the eigenvalue problem.
 
-   Collective on EPS and Mat
+   Collective on eps
 
    Input Parameters:
 +  eps - the eigenproblem solver context
@@ -350,7 +388,7 @@ PetscErrorCode EPSSetOperators(EPS eps,Mat A,Mat B)
 /*@
    EPSGetOperators - Gets the matrices associated with the eigensystem.
 
-   Collective on EPS and Mat
+   Collective on eps
 
    Input Parameter:
 .  eps - the EPS context
@@ -383,11 +421,11 @@ PetscErrorCode EPSGetOperators(EPS eps,Mat *A,Mat *B)
   PetscFunctionReturn(0);
 }
 
-/*@
+/*@C
    EPSSetDeflationSpace - Specify a basis of vectors that constitute the deflation
    space.
 
-   Collective on EPS and Vec
+   Collective on eps
 
    Input Parameter:
 +  eps - the eigenproblem solver context
@@ -410,7 +448,7 @@ PetscErrorCode EPSGetOperators(EPS eps,Mat *A,Mat *B)
 
 .seealso: EPSSetInitialSpace()
 @*/
-PetscErrorCode EPSSetDeflationSpace(EPS eps,PetscInt n,Vec *v)
+PetscErrorCode EPSSetDeflationSpace(EPS eps,PetscInt n,Vec v[])
 {
   PetscErrorCode ierr;
 
@@ -418,6 +456,10 @@ PetscErrorCode EPSSetDeflationSpace(EPS eps,PetscInt n,Vec *v)
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
   PetscValidLogicalCollectiveInt(eps,n,2);
   if (n<0) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"Argument n out of range");
+  if (n>0) {
+    PetscValidPointer(v,3);
+    PetscValidHeaderSpecific(*v,VEC_CLASSID,3);
+  }
   ierr = SlepcBasisReference_Private(n,v,&eps->nds,&eps->defl);CHKERRQ(ierr);
   if (n>0) eps->state = EPS_STATE_INITIAL;
   PetscFunctionReturn(0);
@@ -427,7 +469,7 @@ PetscErrorCode EPSSetDeflationSpace(EPS eps,PetscInt n,Vec *v)
    EPSSetInitialSpace - Specify a basis of vectors that constitute the initial
    space, that is, the subspace from which the solver starts to iterate.
 
-   Collective on EPS and Vec
+   Collective on eps
 
    Input Parameter:
 +  eps - the eigenproblem solver context
@@ -449,9 +491,9 @@ PetscErrorCode EPSSetDeflationSpace(EPS eps,PetscInt n,Vec *v)
 
    Level: intermediate
 
-.seealso: EPSSetDeflationSpace()
+.seealso: EPSSetLeftInitialSpace(), EPSSetDeflationSpace()
 @*/
-PetscErrorCode EPSSetInitialSpace(EPS eps,PetscInt n,Vec *is)
+PetscErrorCode EPSSetInitialSpace(EPS eps,PetscInt n,Vec is[])
 {
   PetscErrorCode ierr;
 
@@ -459,11 +501,54 @@ PetscErrorCode EPSSetInitialSpace(EPS eps,PetscInt n,Vec *is)
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
   PetscValidLogicalCollectiveInt(eps,n,2);
   if (n<0) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"Argument n cannot be negative");
+  if (n>0) {
+    PetscValidPointer(is,3);
+    PetscValidHeaderSpecific(*is,VEC_CLASSID,3);
+  }
   ierr = SlepcBasisReference_Private(n,is,&eps->nini,&eps->IS);CHKERRQ(ierr);
   if (n>0) eps->state = EPS_STATE_INITIAL;
   PetscFunctionReturn(0);
 }
 
+/*@C
+   EPSSetLeftInitialSpace - Specify a basis of vectors that constitute the left
+   initial space, used by two-sided solvers to start the left subspace.
+
+   Collective on eps
+
+   Input Parameter:
++  eps - the eigenproblem solver context
+.  n   - number of vectors
+-  isl - set of basis vectors of the left initial space
+
+   Notes:
+   Left initial vectors are used to initiate the left search space in two-sided
+   eigensolvers. Users should pass here an approximation of the left eigenspace,
+   if available.
+
+   The same comments in EPSSetInitialSpace() are applicable here.
+
+   Level: intermediate
+
+.seealso: EPSSetInitialSpace(), EPSSetTwoSided()
+@*/
+PetscErrorCode EPSSetLeftInitialSpace(EPS eps,PetscInt n,Vec isl[])
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
+  PetscValidLogicalCollectiveInt(eps,n,2);
+  if (n<0) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"Argument n cannot be negative");
+  if (n>0) {
+    PetscValidPointer(isl,3);
+    PetscValidHeaderSpecific(*isl,VEC_CLASSID,3);
+  }
+  ierr = SlepcBasisReference_Private(n,isl,&eps->ninil,&eps->ISL);CHKERRQ(ierr);
+  if (n>0) eps->state = EPS_STATE_INITIAL;
+  PetscFunctionReturn(0);
+}
+
 /*
   EPSSetDimensions_Default - Set reasonable values for ncv, mpd if not set
   by the user. This is called at setup.
@@ -498,7 +583,7 @@ PetscErrorCode EPSSetDimensions_Default(EPS eps,PetscInt nev,PetscInt *ncv,Petsc
    EPSAllocateSolution - Allocate memory storage for common variables such
    as eigenvalues and eigenvectors.
 
-   Collective on EPS
+   Collective on eps
 
    Input Parameters:
 +  eps   - eigensolver context
@@ -506,7 +591,7 @@ PetscErrorCode EPSSetDimensions_Default(EPS eps,PetscInt nev,PetscInt *ncv,Petsc
            working basis slightly larger than ncv
 
    Developers Note:
-   This is PETSC_EXTERN because it may be required by user plugin EPS
+   This is SLEPC_EXTERN because it may be required by user plugin EPS
    implementations.
 
    Level: developer
@@ -554,6 +639,12 @@ PetscErrorCode EPSAllocateSolution(EPS eps,PetscInt extra)
   } else {
     ierr = BVResize(eps->V,requested,PETSC_FALSE);CHKERRQ(ierr);
   }
+
+  /* allocate W */
+  if (eps->twosided) {
+    ierr = BVDestroy(&eps->W);CHKERRQ(ierr);
+    ierr = BVDuplicate(eps->V,&eps->W);CHKERRQ(ierr);
+  }
   PetscFunctionReturn(0);
 }
 
