@@ -1,7 +1,7 @@
 !
 !  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 !  SLEPc - Scalable Library for Eigenvalue Problem Computations
-!  Copyright (c) 2002-2018, Universitat Politecnica de Valencia, Spain
+!  Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
 !
 !  This file is part of SLEPc.
 !  SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -13,20 +13,16 @@
 #include "slepc/finclude/slepcmfn.h"
 
       type tMFN
-        PetscFortranAddr:: v
+        PetscFortranAddr:: v PETSC_FORTRAN_TYPE_INITIALIZE
       end type tMFN
 
-      PetscEnum MFN_CONVERGED_TOL
-      PetscEnum MFN_CONVERGED_ITS
-      PetscEnum MFN_DIVERGED_ITS
-      PetscEnum MFN_DIVERGED_BREAKDOWN
-      PetscEnum MFN_CONVERGED_ITERATING
+      MFN, parameter :: SLEPC_NULL_MFN = tMFN(0)
 
-      parameter (MFN_CONVERGED_TOL          =  1)
-      parameter (MFN_CONVERGED_ITS          =  2)
-      parameter (MFN_DIVERGED_ITS           = -1)
-      parameter (MFN_DIVERGED_BREAKDOWN     = -2)
-      parameter (MFN_CONVERGED_ITERATING    =  0)
+      PetscEnum, parameter :: MFN_CONVERGED_TOL          =  1
+      PetscEnum, parameter :: MFN_CONVERGED_ITS          =  2
+      PetscEnum, parameter :: MFN_DIVERGED_ITS           = -1
+      PetscEnum, parameter :: MFN_DIVERGED_BREAKDOWN     = -2
+      PetscEnum, parameter :: MFN_CONVERGED_ITERATING    =  0
 
 !
 !   Possible arguments to MFNMonitorSet()
