@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -55,8 +55,11 @@ PetscErrorCode NEPSetUp_Interpol(NEP nep)
   if (!ctx->pep) { ierr = NEPInterpolGetPEP(nep,&ctx->pep);CHKERRQ(ierr); }
   ierr = PEPSetBasis(ctx->pep,PEP_BASIS_CHEBYSHEV1);CHKERRQ(ierr);
   ierr = PEPSetWhichEigenpairs(ctx->pep,PEP_TARGET_MAGNITUDE);CHKERRQ(ierr);
-  ierr = PEPGetST(ctx->pep,&st);CHKERRQ(ierr);
-  ierr = STSetType(st,STSINVERT);CHKERRQ(ierr);
+  ierr = PetscObjectTypeCompare((PetscObject)ctx->pep,PEPJD,&flg);CHKERRQ(ierr);
+  if (!flg) {
+    ierr = PEPGetST(ctx->pep,&st);CHKERRQ(ierr);
+    ierr = STSetType(st,STSINVERT);CHKERRQ(ierr);
+  }
   ierr = PEPSetDimensions(ctx->pep,nep->nev,nep->ncv?nep->ncv:PETSC_DEFAULT,nep->mpd?nep->mpd:PETSC_DEFAULT);CHKERRQ(ierr);
   ierr = PEPGetTolerances(ctx->pep,&tol,&its);CHKERRQ(ierr);
   if (tol==PETSC_DEFAULT) tol = (nep->tol==PETSC_DEFAULT)?SLEPC_DEFAULT_TOL:nep->tol;
@@ -118,7 +121,7 @@ PetscErrorCode NEPSolve_Interpol(NEP nep)
   PetscScalar    *x,*fx,t;
   PetscReal      *cs,a,b,s,aprox,aprox0=1.0,*matnorm;
   PetscInt       i,j,k,deg=ctx->maxdeg;
-  PetscBool      hasmnorm;
+  PetscBool      hasmnorm=PETSC_FALSE;
   Vec            vr,vi=NULL;
 
   PetscFunctionBegin;
@@ -155,7 +158,7 @@ PetscErrorCode NEPSolve_Interpol(NEP nep)
       ierr = MatAXPY(A[k],t,nep->A[j],nep->mstr);CHKERRQ(ierr);
     }
     if (k==0) aprox0 = aprox;
-    if (aprox/aprox0<ctx->tol) { ctx->deg = k; deg = k; break; }
+    if (k>1 && aprox/aprox0<ctx->tol) { ctx->deg = k; deg = k; break; }
   }
 
   ierr = PEPSetOperators(ctx->pep,deg+1,A);CHKERRQ(ierr);
@@ -171,7 +174,7 @@ PetscErrorCode NEPSolve_Interpol(NEP nep)
   ierr = PEPGetConvergedReason(ctx->pep,(PEPConvergedReason*)&nep->reason);CHKERRQ(ierr);
   ierr = BVSetActiveColumns(nep->V,0,nep->nconv);CHKERRQ(ierr);
   ierr = BVCreateVec(nep->V,&vr);CHKERRQ(ierr);
-#if !defined(PETSC_USE_COMPLEX)   
+#if !defined(PETSC_USE_COMPLEX)
   ierr = VecDuplicate(vr,&vi);CHKERRQ(ierr);
 #endif
   s = 2.0/(b-a);
@@ -181,7 +184,7 @@ PetscErrorCode NEPSolve_Interpol(NEP nep)
     nep->eigr[i] += (a+b)/2.0;
     nep->eigi[i] /= s;
     ierr = BVInsertVec(nep->V,i,vr);CHKERRQ(ierr);
-#if !defined(PETSC_USE_COMPLEX)   
+#if !defined(PETSC_USE_COMPLEX)
     if (nep->eigi[i]!=0.0) {
       ierr = BVInsertVec(nep->V,++i,vi);CHKERRQ(ierr);
     }
@@ -278,7 +281,7 @@ static PetscErrorCode NEPInterpolSetInterpolation_Interpol(NEP nep,PetscReal tol
    NEPInterpolSetInterpolation - Sets the tolerance and maximum degree when building
    the interpolation polynomial.
 
-   Collective on NEP
+   Collective on nep
 
    Input Parameters:
 +  nep - nonlinear eigenvalue solver
@@ -363,7 +366,7 @@ static PetscErrorCode NEPInterpolSetPEP_Interpol(NEP nep,PEP pep)
    NEPInterpolSetPEP - Associate a polynomial eigensolver object (PEP) to the
    nonlinear eigenvalue solver.
 
-   Collective on NEP
+   Collective on nep
 
    Input Parameters:
 +  nep - nonlinear eigenvalue solver
@@ -393,10 +396,11 @@ static PetscErrorCode NEPInterpolGetPEP_Interpol(NEP nep,PEP *pep)
   PetscFunctionBegin;
   if (!ctx->pep) {
     ierr = PEPCreate(PetscObjectComm((PetscObject)nep),&ctx->pep);CHKERRQ(ierr);
+    ierr = PetscObjectIncrementTabLevel((PetscObject)ctx->pep,(PetscObject)nep,1);CHKERRQ(ierr);
     ierr = PEPSetOptionsPrefix(ctx->pep,((PetscObject)nep)->prefix);CHKERRQ(ierr);
     ierr = PEPAppendOptionsPrefix(ctx->pep,"nep_interpol_");CHKERRQ(ierr);
-    ierr = PetscObjectIncrementTabLevel((PetscObject)ctx->pep,(PetscObject)nep,1);CHKERRQ(ierr);
     ierr = PetscLogObjectParent((PetscObject)nep,(PetscObject)ctx->pep);CHKERRQ(ierr);
+    ierr = PetscObjectSetOptions((PetscObject)ctx->pep,((PetscObject)nep)->options);CHKERRQ(ierr);
     ierr = PEPMonitorSet(ctx->pep,PEPMonitor_Interpol,nep,NULL);CHKERRQ(ierr);
   }
   *pep = ctx->pep;
@@ -474,7 +478,7 @@ PetscErrorCode NEPDestroy_Interpol(NEP nep)
   PetscFunctionReturn(0);
 }
 
-PETSC_EXTERN PetscErrorCode NEPCreate_Interpol(NEP nep)
+SLEPC_EXTERN PetscErrorCode NEPCreate_Interpol(NEP nep)
 {
   PetscErrorCode ierr;
   NEP_INTERPOL   *ctx;
