@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2018, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -17,7 +17,7 @@
 /*@C
    NEPView - Prints the NEP data structure.
 
-   Collective on NEP
+   Collective on nep
 
    Input Parameters:
 +  nep - the nonlinear eigenproblem solver context
@@ -38,28 +38,26 @@
    PetscViewerASCIIOpen() - output to a specified file.
 
    Level: beginner
-
-.seealso: PetscViewerASCIIOpen()
 @*/
 PetscErrorCode NEPView(NEP nep,PetscViewer viewer)
 {
   PetscErrorCode ierr;
   const char     *type=NULL;
   char           str[50];
-  PetscInt       i,tabs;
-  PetscBool      isascii,istrivial,nods;
+  PetscInt       i;
+  PetscBool      isascii,istrivial;
   PetscViewer    sviewer;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
-  if (!viewer) viewer = PETSC_VIEWER_STDOUT_(PetscObjectComm((PetscObject)nep));
+  if (!viewer) {
+    ierr = PetscViewerASCIIGetStdout(PetscObjectComm((PetscObject)nep),&viewer);CHKERRQ(ierr);
+  }
   PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,2);
   PetscCheckSameComm(nep,1,viewer,2);
 
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
   if (isascii) {
-    ierr = PetscViewerASCIIGetTab(viewer,&tabs);CHKERRQ(ierr);
-    ierr = PetscViewerASCIISetTab(viewer,((PetscObject)nep)->tablevel);CHKERRQ(ierr);
     ierr = PetscObjectPrintClassNamePrefixType((PetscObject)nep,viewer);CHKERRQ(ierr);
     if (nep->ops->view) {
       ierr = PetscViewerASCIIPushTab(viewer);CHKERRQ(ierr);
@@ -81,9 +79,6 @@ PetscErrorCode NEPView(NEP nep,PetscViewer viewer)
       case NEP_USER_INTERFACE_SPLIT:
         ierr = PetscViewerASCIIPrintf(viewer,"  nonlinear operator in split form, with %D terms\n",nep->nt);CHKERRQ(ierr);
         break;
-      case NEP_USER_INTERFACE_DERIVATIVES:
-        ierr = PetscViewerASCIIPrintf(viewer,"  nonlinear operator from user callbacks for the successive derivatives\n");CHKERRQ(ierr);
-        break;
       }
     } else {
       ierr = PetscViewerASCIIPrintf(viewer,"  nonlinear operator not specified yet\n");CHKERRQ(ierr);
@@ -129,6 +124,9 @@ PetscErrorCode NEPView(NEP nep,PetscViewer viewer)
         break;
     }
     ierr = PetscViewerASCIIUseTabs(viewer,PETSC_TRUE);CHKERRQ(ierr);
+    if (nep->twosided) {
+      ierr = PetscViewerASCIIPrintf(viewer,"  using two-sided variant (for left eigenvectors)\n");CHKERRQ(ierr);
+    }
     ierr = PetscViewerASCIIPrintf(viewer,"  number of eigenvalues (nev): %D\n",nep->nev);CHKERRQ(ierr);
     ierr = PetscViewerASCIIPrintf(viewer,"  number of column vectors (ncv): %D\n",nep->ncv);CHKERRQ(ierr);
     ierr = PetscViewerASCIIPrintf(viewer,"  maximum dimension of projected problem (mpd): %D\n",nep->mpd);CHKERRQ(ierr);
@@ -165,7 +163,6 @@ PetscErrorCode NEPView(NEP nep,PetscViewer viewer)
     if (nep->nini) {
       ierr = PetscViewerASCIIPrintf(viewer,"  dimension of user-provided initial space: %D\n",PetscAbs(nep->nini));CHKERRQ(ierr);
     }
-    ierr = PetscViewerASCIISetTab(viewer,tabs);CHKERRQ(ierr);
   } else {
     if (nep->ops->view) {
       ierr = (*nep->ops->view)(nep,viewer);CHKERRQ(ierr);
@@ -177,13 +174,13 @@ PetscErrorCode NEPView(NEP nep,PetscViewer viewer)
   if (!nep->rg) { ierr = NEPGetRG(nep,&nep->rg);CHKERRQ(ierr); }
   ierr = RGIsTrivial(nep->rg,&istrivial);CHKERRQ(ierr);
   if (!istrivial) { ierr = RGView(nep->rg,viewer);CHKERRQ(ierr); }
-  ierr = PetscObjectTypeCompareAny((PetscObject)nep,&nods,NEPRII,NEPSLP,NEPINTERPOL,"");CHKERRQ(ierr);
-  if (!nods) {
+  if (nep->useds) {
     if (!nep->ds) { ierr = NEPGetDS(nep,&nep->ds);CHKERRQ(ierr); }
     ierr = DSView(nep->ds,viewer);CHKERRQ(ierr);
   }
   ierr = PetscViewerPopFormat(viewer);CHKERRQ(ierr);
   if (nep->refine!=NEP_REFINE_NONE) {
+    ierr = PetscViewerASCIIPushTab(viewer);CHKERRQ(ierr);
     if (nep->npart>1) {
       if (nep->refinesubc->color==0) {
         ierr = PetscViewerASCIIGetStdout(PetscSubcommChild(nep->refinesubc),&sviewer);CHKERRQ(ierr);
@@ -192,16 +189,41 @@ PetscErrorCode NEPView(NEP nep,PetscViewer viewer)
     } else {
       ierr = KSPView(nep->refineksp,viewer);CHKERRQ(ierr);
     }
+    ierr = PetscViewerASCIIPopTab(viewer);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
+/*@C
+   NEPViewFromOptions - View from options
+
+   Collective on NEP
+
+   Input Parameters:
++  nep  - the nonlinear eigensolver context
+.  obj  - optional object
+-  name - command line option
+
+   Level: intermediate
+
+.seealso: NEPView(), NEPCreate()
+@*/
+PetscErrorCode NEPViewFromOptions(NEP nep,PetscObject obj,const char name[])
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
+  ierr = PetscObjectViewFromOptions((PetscObject)nep,obj,name);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
 /*@C
    NEPReasonView - Displays the reason a NEP solve converged or diverged.
 
-   Collective on NEP
+   Collective on nep
 
-   Parameter:
+   Input Parameters:
 +  nep - the nonlinear eigensolver context
 -  viewer - the viewer to display the reason
 
@@ -235,9 +257,9 @@ PetscErrorCode NEPReasonView(NEP nep,PetscViewer viewer)
    NEPReasonViewFromOptions - Processes command line options to determine if/how
    the NEP converged reason is to be viewed.
 
-   Collective on NEP
+   Collective on nep
 
-   Input Parameters:
+   Input Parameter:
 .  nep - the nonlinear eigensolver context
 
    Level: developer
@@ -253,7 +275,7 @@ PetscErrorCode NEPReasonViewFromOptions(NEP nep)
   PetscFunctionBegin;
   if (incall) PetscFunctionReturn(0);
   incall = PETSC_TRUE;
-  ierr = PetscOptionsGetViewer(PetscObjectComm((PetscObject)nep),((PetscObject)nep)->prefix,"-nep_converged_reason",&viewer,&format,&flg);CHKERRQ(ierr);
+  ierr = PetscOptionsGetViewer(PetscObjectComm((PetscObject)nep),((PetscObject)nep)->options,((PetscObject)nep)->prefix,"-nep_converged_reason",&viewer,&format,&flg);CHKERRQ(ierr);
   if (flg) {
     ierr = PetscViewerPushFormat(viewer,format);CHKERRQ(ierr);
     ierr = NEPReasonView(nep,viewer);CHKERRQ(ierr);
@@ -296,7 +318,7 @@ static PetscErrorCode NEPErrorView_ASCII(NEP nep,NEPErrorType etype,PetscViewer
     ierr = PetscViewerASCIIPrintf(viewer,"\n     ");CHKERRQ(ierr);
     for (j=0;j<PetscMin(8,nvals-8*i);j++) {
       k = nep->perm[8*i+j];
-      ierr = SlepcPrintEigenvalueASCII(nep->eigr[k],nep->eigi[k]);CHKERRQ(ierr);
+      ierr = SlepcPrintEigenvalueASCII(viewer,nep->eigr[k],nep->eigi[k]);CHKERRQ(ierr);
       if (8*i+j+1<nvals) { ierr = PetscViewerASCIIPrintf(viewer,", ");CHKERRQ(ierr); }
     }
   }
@@ -369,7 +391,7 @@ static PetscErrorCode NEPErrorView_MATLAB(NEP nep,NEPErrorType etype,PetscViewer
    NEPErrorView - Displays the errors associated with the computed solution
    (as well as the eigenvalues).
 
-   Collective on NEP
+   Collective on nep
 
    Input Parameters:
 +  nep    - the nonlinear eigensolver context
@@ -399,7 +421,9 @@ PetscErrorCode NEPErrorView(NEP nep,NEPErrorType etype,PetscViewer viewer)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
-  if (!viewer) viewer = PETSC_VIEWER_STDOUT_(PetscObjectComm((PetscObject)nep));
+  if (!viewer) {
+    ierr = PetscViewerASCIIGetStdout(PetscObjectComm((PetscObject)nep),&viewer);CHKERRQ(ierr);
+  }
   PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,2);
   PetscCheckSameComm(nep,1,viewer,2);
   NEPCheckSolved(nep,1);
@@ -428,9 +452,9 @@ PetscErrorCode NEPErrorView(NEP nep,NEPErrorType etype,PetscViewer viewer)
    NEPErrorViewFromOptions - Processes command line options to determine if/how
    the errors of the computed solution are to be viewed.
 
-   Collective on NEP
+   Collective on nep
 
-   Input Parameters:
+   Input Parameter:
 .  nep - the nonlinear eigensolver context
 
    Level: developer
@@ -446,21 +470,21 @@ PetscErrorCode NEPErrorViewFromOptions(NEP nep)
   PetscFunctionBegin;
   if (incall) PetscFunctionReturn(0);
   incall = PETSC_TRUE;
-  ierr = PetscOptionsGetViewer(PetscObjectComm((PetscObject)nep),((PetscObject)nep)->prefix,"-nep_error_absolute",&viewer,&format,&flg);CHKERRQ(ierr);
+  ierr = PetscOptionsGetViewer(PetscObjectComm((PetscObject)nep),((PetscObject)nep)->options,((PetscObject)nep)->prefix,"-nep_error_absolute",&viewer,&format,&flg);CHKERRQ(ierr);
   if (flg) {
     ierr = PetscViewerPushFormat(viewer,format);CHKERRQ(ierr);
     ierr = NEPErrorView(nep,NEP_ERROR_ABSOLUTE,viewer);CHKERRQ(ierr);
     ierr = PetscViewerPopFormat(viewer);CHKERRQ(ierr);
     ierr = PetscViewerDestroy(&viewer);CHKERRQ(ierr);
   }
-  ierr = PetscOptionsGetViewer(PetscObjectComm((PetscObject)nep),((PetscObject)nep)->prefix,"-nep_error_relative",&viewer,&format,&flg);CHKERRQ(ierr);
+  ierr = PetscOptionsGetViewer(PetscObjectComm((PetscObject)nep),((PetscObject)nep)->options,((PetscObject)nep)->prefix,"-nep_error_relative",&viewer,&format,&flg);CHKERRQ(ierr);
   if (flg) {
     ierr = PetscViewerPushFormat(viewer,format);CHKERRQ(ierr);
     ierr = NEPErrorView(nep,NEP_ERROR_RELATIVE,viewer);CHKERRQ(ierr);
     ierr = PetscViewerPopFormat(viewer);CHKERRQ(ierr);
     ierr = PetscViewerDestroy(&viewer);CHKERRQ(ierr);
   }
-  ierr = PetscOptionsGetViewer(PetscObjectComm((PetscObject)nep),((PetscObject)nep)->prefix,"-nep_error_backward",&viewer,&format,&flg);CHKERRQ(ierr);
+  ierr = PetscOptionsGetViewer(PetscObjectComm((PetscObject)nep),((PetscObject)nep)->options,((PetscObject)nep)->prefix,"-nep_error_backward",&viewer,&format,&flg);CHKERRQ(ierr);
   if (flg) {
     ierr = PetscViewerPushFormat(viewer,format);CHKERRQ(ierr);
     ierr = NEPErrorView(nep,NEP_ERROR_BACKWARD,viewer);CHKERRQ(ierr);
@@ -511,7 +535,7 @@ static PetscErrorCode NEPValuesView_ASCII(NEP nep,PetscViewer viewer)
   for (i=0;i<nep->nconv;i++) {
     k = nep->perm[i];
     ierr = PetscViewerASCIIPrintf(viewer,"   ");CHKERRQ(ierr);
-    ierr = SlepcPrintEigenvalueASCII(nep->eigr[k],nep->eigi[k]);CHKERRQ(ierr);
+    ierr = SlepcPrintEigenvalueASCII(viewer,nep->eigr[k],nep->eigi[k]);CHKERRQ(ierr);
     ierr = PetscViewerASCIIPrintf(viewer,"\n");CHKERRQ(ierr);
   }
   ierr = PetscViewerASCIIPrintf(viewer,"\n");CHKERRQ(ierr);
@@ -550,7 +574,7 @@ static PetscErrorCode NEPValuesView_MATLAB(NEP nep,PetscViewer viewer)
 /*@C
    NEPValuesView - Displays the computed eigenvalues in a viewer.
 
-   Collective on NEP
+   Collective on nep
 
    Input Parameters:
 +  nep    - the nonlinear eigensolver context
@@ -571,7 +595,9 @@ PetscErrorCode NEPValuesView(NEP nep,PetscViewer viewer)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
-  if (!viewer) viewer = PETSC_VIEWER_STDOUT_(PetscObjectComm((PetscObject)nep));
+  if (!viewer) {
+    ierr = PetscViewerASCIIGetStdout(PetscObjectComm((PetscObject)nep),&viewer);CHKERRQ(ierr);
+  }
   PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,2);
   PetscCheckSameComm(nep,1,viewer,2);
   NEPCheckSolved(nep,1);
@@ -601,9 +627,9 @@ PetscErrorCode NEPValuesView(NEP nep,PetscViewer viewer)
    NEPValuesViewFromOptions - Processes command line options to determine if/how
    the computed eigenvalues are to be viewed.
 
-   Collective on NEP
+   Collective on nep
 
-   Input Parameters:
+   Input Parameter:
 .  nep - the nonlinear eigensolver context
 
    Level: developer
@@ -619,7 +645,7 @@ PetscErrorCode NEPValuesViewFromOptions(NEP nep)
   PetscFunctionBegin;
   if (incall) PetscFunctionReturn(0);
   incall = PETSC_TRUE;
-  ierr = PetscOptionsGetViewer(PetscObjectComm((PetscObject)nep),((PetscObject)nep)->prefix,"-nep_view_values",&viewer,&format,&flg);CHKERRQ(ierr);
+  ierr = PetscOptionsGetViewer(PetscObjectComm((PetscObject)nep),((PetscObject)nep)->options,((PetscObject)nep)->prefix,"-nep_view_values",&viewer,&format,&flg);CHKERRQ(ierr);
   if (flg) {
     ierr = PetscViewerPushFormat(viewer,format);CHKERRQ(ierr);
     ierr = NEPValuesView(nep,viewer);CHKERRQ(ierr);
@@ -633,20 +659,24 @@ PetscErrorCode NEPValuesViewFromOptions(NEP nep)
 /*@C
    NEPVectorsView - Outputs computed eigenvectors to a viewer.
 
-   Collective on NEP
+   Collective on nep
 
-   Parameter:
+   Input Parameters:
 +  nep    - the nonlinear eigensolver context
 -  viewer - the viewer
 
    Options Database Keys:
 .  -nep_view_vectors - output eigenvectors.
 
-   Note:
+   Notes:
    If PETSc was configured with real scalars, complex conjugate eigenvectors
    will be viewed as two separate real vectors, one containing the real part
    and another one containing the imaginary part.
 
+   If left eigenvectors were computed with a two-sided eigensolver, the right
+   and left eigenvectors are interleaved, that is, the vectors are output in
+   the following order: X0, Y0, X1, Y1, X2, Y2, ...
+
    Level: intermediate
 
 .seealso: NEPSolve(), NEPValuesView(), NEPErrorView()
@@ -662,7 +692,9 @@ PetscErrorCode NEPVectorsView(NEP nep,PetscViewer viewer)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
-  if (!viewer) viewer = PETSC_VIEWER_STDOUT_(PetscObjectComm((PetscObject)nep));
+  if (!viewer) {
+    ierr = PetscViewerASCIIGetStdout(PetscObjectComm((PetscObject)nep),&viewer);CHKERRQ(ierr);
+  }
   PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,2);
   PetscCheckSameComm(nep,1,viewer,2);
   NEPCheckSolved(nep,1);
@@ -671,11 +703,18 @@ PetscErrorCode NEPVectorsView(NEP nep,PetscViewer viewer)
     ierr = NEPComputeVectors(nep);CHKERRQ(ierr);
     for (i=0;i<nep->nconv;i++) {
       k = nep->perm[i];
-      ierr = PetscSNPrintf(vname,NMLEN,"V%d_%s",(int)i,ename);CHKERRQ(ierr);
+      ierr = PetscSNPrintf(vname,NMLEN,"X%d_%s",(int)i,ename);CHKERRQ(ierr);
       ierr = BVGetColumn(nep->V,k,&x);CHKERRQ(ierr);
       ierr = PetscObjectSetName((PetscObject)x,vname);CHKERRQ(ierr);
       ierr = VecView(x,viewer);CHKERRQ(ierr);
       ierr = BVRestoreColumn(nep->V,k,&x);CHKERRQ(ierr);
+      if (nep->twosided) {
+        ierr = PetscSNPrintf(vname,NMLEN,"Y%d_%s",(int)i,ename);CHKERRQ(ierr);
+        ierr = BVGetColumn(nep->W,k,&x);CHKERRQ(ierr);
+        ierr = PetscObjectSetName((PetscObject)x,vname);CHKERRQ(ierr);
+        ierr = VecView(x,viewer);CHKERRQ(ierr);
+        ierr = BVRestoreColumn(nep->W,k,&x);CHKERRQ(ierr);
+      }
     }
   }
   PetscFunctionReturn(0);
@@ -685,9 +724,9 @@ PetscErrorCode NEPVectorsView(NEP nep,PetscViewer viewer)
    NEPVectorsViewFromOptions - Processes command line options to determine if/how
    the computed eigenvectors are to be viewed.
 
-   Collective on NEP
+   Collective on nep
 
-   Input Parameters:
+   Input Parameter:
 .  nep - the nonlinear eigensolver context
 
    Level: developer
@@ -703,7 +742,7 @@ PetscErrorCode NEPVectorsViewFromOptions(NEP nep)
   PetscFunctionBegin;
   if (incall) PetscFunctionReturn(0);
   incall = PETSC_TRUE;
-  ierr = PetscOptionsGetViewer(PetscObjectComm((PetscObject)nep),((PetscObject)nep)->prefix,"-nep_view_vectors",&viewer,&format,&flg);CHKERRQ(ierr);
+  ierr = PetscOptionsGetViewer(PetscObjectComm((PetscObject)nep),((PetscObject)nep)->options,((PetscObject)nep)->prefix,"-nep_view_vectors",&viewer,&format,&flg);CHKERRQ(ierr);
   if (flg) {
     ierr = PetscViewerPushFormat(viewer,format);CHKERRQ(ierr);
     ierr = NEPVectorsView(nep,viewer);CHKERRQ(ierr);
