@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2018, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -40,36 +40,85 @@ static const char citation[] =
   "   doi = \"https://doi.org/10.1007/s10543-016-0601-5\"\n"
   "}\n";
 
+typedef struct {
+  PetscReal   scal[2];
+  Mat         A[2];
+  Vec         t;
+} PEP_STOAR_MATSHELL;
 
-PetscErrorCode MatMult_Func(Mat A,Vec x,Vec y)
+static PetscErrorCode MatMult_STOAR(Mat A,Vec x,Vec y)
 {
-  PetscErrorCode ierr;
-  ShellMatCtx    *ctx;
+  PetscErrorCode     ierr;
+  PEP_STOAR_MATSHELL *ctx;
 
   PetscFunctionBegin;
   ierr = MatShellGetContext(A,&ctx);CHKERRQ(ierr);
-  ierr = MatMult(ctx->A,x,y);CHKERRQ(ierr);
-  ierr = VecScale(y,ctx->scal);CHKERRQ(ierr);
+  ierr = MatMult(ctx->A[0],x,y);CHKERRQ(ierr);
+  ierr = VecScale(y,ctx->scal[0]);CHKERRQ(ierr);
+  if (ctx->scal[1]) {
+    ierr = MatMult(ctx->A[1],x,ctx->t);CHKERRQ(ierr);
+    ierr = VecAXPY(y,ctx->scal[1],ctx->t);CHKERRQ(ierr);
+  }
   PetscFunctionReturn(0);
 }
 
-PetscErrorCode MatDestroy_Func(Mat A)
+static PetscErrorCode MatDestroy_STOAR(Mat A)
 {
-  ShellMatCtx    *ctx;
-  PetscErrorCode ierr;
+  PEP_STOAR_MATSHELL *ctx;
+  PetscErrorCode     ierr;
 
   PetscFunctionBegin;
   ierr = MatShellGetContext(A,(void**)&ctx);CHKERRQ(ierr);
+  ierr = VecDestroy(&ctx->t);CHKERRQ(ierr);
   ierr = PetscFree(ctx);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
+PetscErrorCode PEPSTOARSetUpInnerMatrix(PEP pep,Mat *B)
+{
+  Mat                pB[4],Bs[3],D[3];
+  PetscInt           i,j,n,m;
+  PEP_STOAR_MATSHELL *ctxMat[3];
+  PEP_STOAR          *ctx=(PEP_STOAR*)pep->data;
+  PetscErrorCode     ierr;
+
+  PetscFunctionBegin;
+  for (i=0;i<3;i++) {
+    ierr = STGetMatrixTransformed(pep->st,i,&D[i]);CHKERRQ(ierr); /* D[2] = M */
+  }
+  ierr = MatGetLocalSize(D[2],&m,&n);CHKERRQ(ierr);
+
+  for (j=0;j<3;j++) {
+    ierr = PetscNew(ctxMat+j);CHKERRQ(ierr);
+    ierr = MatCreateShell(PetscObjectComm((PetscObject)pep),m,n,PETSC_DETERMINE,PETSC_DETERMINE,ctxMat[j],&Bs[j]);CHKERRQ(ierr);
+    ierr = MatShellSetOperation(Bs[j],MATOP_MULT,(void(*)(void))MatMult_STOAR);CHKERRQ(ierr);
+    ierr = MatShellSetOperation(Bs[j],MATOP_DESTROY,(void(*)(void))MatDestroy_STOAR);CHKERRQ(ierr);
+  }
+  for (i=0;i<4;i++) pB[i] = NULL;
+  if (ctx->alpha) {
+    ctxMat[0]->A[0] = D[0]; ctxMat[0]->scal[0] = ctx->alpha; ctxMat[0]->scal[1] = 0.0;
+    ctxMat[2]->A[0] = D[2]; ctxMat[2]->scal[0] = -ctx->alpha*pep->sfactor*pep->sfactor; ctxMat[2]->scal[1] = 0.0;
+    pB[0] = Bs[0]; pB[3] = Bs[2];
+  }
+  if (ctx->beta) {
+    i = (ctx->alpha)?1:0;
+    ctxMat[0]->scal[1] = 0.0;
+    ctxMat[0]->A[i] = D[1]; ctxMat[0]->scal[i] = -ctx->beta*pep->sfactor;
+    ctxMat[1]->A[0] = D[2]; ctxMat[1]->scal[0] = -ctx->beta*pep->sfactor*pep->sfactor; ctxMat[1]->scal[1] = 0.0;
+    pB[0] = Bs[0]; pB[1] = pB[2] = Bs[1];
+  }
+  ierr = BVCreateVec(pep->V,&ctxMat[0]->t);CHKERRQ(ierr);
+  ierr = MatCreateNest(PetscObjectComm((PetscObject)pep),2,NULL,2,NULL,pB,B);CHKERRQ(ierr);
+  for (j=0;j<3;j++) { ierr = MatDestroy(&Bs[j]);CHKERRQ(ierr); }
+  PetscFunctionReturn(0);
+}
+
 PetscErrorCode PEPSetUp_STOAR(PEP pep)
 {
   PetscErrorCode    ierr;
   PetscBool         shift,sinv,flg;
-  PEP_TOAR          *ctx = (PEP_TOAR*)pep->data;
-  PetscInt          ld;
+  PEP_STOAR         *ctx = (PEP_STOAR*)pep->data;
+  PetscInt          ld,i;
   PetscReal         eta;
   BVOrthogType      otype;
   BVOrthogBlockType obtype;
@@ -79,6 +128,7 @@ PetscErrorCode PEPSetUp_STOAR(PEP pep)
   if (pep->nmat!=3) SETERRQ(PetscObjectComm((PetscObject)pep),PETSC_ERR_SUP,"Solver only available for quadratic problems");
   if (pep->basis!=PEP_BASIS_MONOMIAL) SETERRQ(PetscObjectComm((PetscObject)pep),PETSC_ERR_SUP,"Solver not implemented for non-monomial bases");
   /* spectrum slicing requires special treatment of default values */
+  ierr = PetscObjectTypeCompare((PetscObject)pep->st,STSINVERT,&sinv);CHKERRQ(ierr);
   if (pep->which==PEP_ALL) {
     if (pep->stopping!=PEPStoppingBasic) SETERRQ(PetscObjectComm((PetscObject)pep),PETSC_ERR_SUP,"Spectrum slicing does not support user-defined stopping test");
     pep->ops->solve = PEPSolve_STOAR_QSlice;
@@ -94,8 +144,19 @@ PetscErrorCode PEPSetUp_STOAR(PEP pep)
     ierr = DSSetType(pep->ds,DSGHIEP);CHKERRQ(ierr);
     ierr = DSSetCompact(pep->ds,PETSC_TRUE);CHKERRQ(ierr);
     ierr = DSAllocate(pep->ds,ld);CHKERRQ(ierr);
+    ierr = PEPBasisCoefficients(pep,pep->pbc);CHKERRQ(ierr);
     ierr = STGetTransform(pep->st,&flg);CHKERRQ(ierr);
-    if (!flg) SETERRQ(PetscObjectComm((PetscObject)pep),PETSC_ERR_SUP,"Solver requires the ST transformation flag set, see STSetTransform()");
+    if (!flg) {
+      ierr = PetscFree(pep->solvematcoeffs);CHKERRQ(ierr);
+      ierr = PetscMalloc1(pep->nmat,&pep->solvematcoeffs);CHKERRQ(ierr);
+      ierr = PetscLogObjectMemory((PetscObject)pep,pep->nmat*sizeof(PetscScalar));CHKERRQ(ierr);
+      if (sinv) {
+        ierr = PEPEvaluateBasis(pep,pep->target,0,pep->solvematcoeffs,NULL);CHKERRQ(ierr);
+      } else {
+        for (i=0;i<pep->nmat-1;i++) pep->solvematcoeffs[i] = 0.0;
+        pep->solvematcoeffs[pep->nmat-1] = 1.0;
+      }
+    }
   }
 
   pep->lineariz = PETSC_TRUE;
@@ -107,7 +168,6 @@ PetscErrorCode PEPSetUp_STOAR(PEP pep)
     else pep->which = PEP_LARGEST_MAGNITUDE;
   }
 
-
   ierr = PEPAllocateSolution(pep,2);CHKERRQ(ierr);
   ierr = PEPSetWorkVecs(pep,4);CHKERRQ(ierr);
   ierr = BVDestroy(&ctx->V);CHKERRQ(ierr);
@@ -123,14 +183,14 @@ PetscErrorCode PEPSetUp_STOAR(PEP pep)
 static PetscErrorCode PEPSTOARrun(PEP pep,PetscReal *a,PetscReal *b,PetscReal *omega,PetscInt k,PetscInt *M,PetscBool *breakdown,PetscBool *symmlost,Vec *t_)
 {
   PetscErrorCode ierr;
-  PEP_TOAR       *ctx = (PEP_TOAR*)pep->data;
+  PEP_STOAR      *ctx = (PEP_STOAR*)pep->data;
   PetscInt       i,j,m=*M,l,lock;
-  PetscInt       lds,d,ld,offq,nqt;
+  PetscInt       lds,d,ld,offq,nqt,ldds;
   Vec            v=t_[0],t=t_[1],q=t_[2];
   PetscReal      norm,sym=0.0,fro=0.0,*f;
-  PetscScalar    *y,*S;
+  PetscScalar    *y,*S,*x,sigma;
   PetscBLASInt   j_,one=1;
-  PetscBool      lindep;
+  PetscBool      lindep,flg,sinvert=PETSC_FALSE;
   Mat            MS;
 
   PetscFunctionBegin;
@@ -140,32 +200,66 @@ static PetscErrorCode PEPSTOARrun(PEP pep,PetscReal *a,PetscReal *b,PetscReal *o
   ierr = BVGetActiveColumns(pep->V,&lock,&nqt);CHKERRQ(ierr);
   lds = d*ld;
   offq = ld;
+  ierr = DSGetLeadingDimension(pep->ds,&ldds);CHKERRQ(ierr);
   *breakdown = PETSC_FALSE; /* ----- */
   ierr = DSGetDimensions(pep->ds,NULL,NULL,&l,NULL,NULL);CHKERRQ(ierr);
   ierr = BVSetActiveColumns(ctx->V,0,m);CHKERRQ(ierr);
   ierr = BVSetActiveColumns(pep->V,0,nqt);CHKERRQ(ierr);
+  ierr = STGetTransform(pep->st,&flg);CHKERRQ(ierr);
+  if (!flg) {
+    /* spectral transformation handled by the solver */
+    ierr = PetscObjectTypeCompareAny((PetscObject)pep->st,&flg,STSINVERT,STSHIFT,"");CHKERRQ(ierr);
+    if (!flg) SETERRQ(PetscObjectComm((PetscObject)pep),PETSC_ERR_SUP,"ST type not supported for TOAR without transforming matrices");
+    ierr = PetscObjectTypeCompare((PetscObject)pep->st,STSINVERT,&sinvert);CHKERRQ(ierr);
+    ierr = STGetShift(pep->st,&sigma);CHKERRQ(ierr);
+  }
   for (j=k;j<m;j++) {
     /* apply operator */
     ierr = BVTensorGetFactors(ctx->V,NULL,&MS);CHKERRQ(ierr);
     ierr = MatDenseGetArray(MS,&S);CHKERRQ(ierr);
     ierr = BVGetColumn(pep->V,nqt,&t);CHKERRQ(ierr);
     ierr = BVMultVec(pep->V,1.0,0.0,v,S+j*lds);CHKERRQ(ierr);
-    ierr = STMatMult(pep->st,0,v,t);CHKERRQ(ierr);
-    ierr = BVMultVec(pep->V,1.0,0.0,v,S+offq+j*lds);CHKERRQ(ierr);
-    ierr = STMatMult(pep->st,1,v,q);CHKERRQ(ierr);
-    ierr = VecAXPY(q,pep->sfactor,t);CHKERRQ(ierr);
-    ierr = STMatSolve(pep->st,q,t);CHKERRQ(ierr);
-    ierr = VecScale(t,-1.0/(pep->sfactor*pep->sfactor));CHKERRQ(ierr);
+    if (!sinvert) {
+      ierr = STMatMult(pep->st,0,v,q);CHKERRQ(ierr);
+      ierr = BVMultVec(pep->V,1.0,0.0,v,S+offq+j*lds);CHKERRQ(ierr);
+      ierr = STMatMult(pep->st,1,v,t);CHKERRQ(ierr);
+      ierr = VecAXPY(q,pep->sfactor,t);CHKERRQ(ierr);
+      if (ctx->beta && ctx->alpha) {
+        ierr = STMatMult(pep->st,2,v,t);CHKERRQ(ierr);
+        ierr = VecAXPY(q,-pep->sfactor*pep->sfactor*ctx->beta/ctx->alpha,t);CHKERRQ(ierr);
+      }
+      ierr = STMatSolve(pep->st,q,t);CHKERRQ(ierr);
+      ierr = VecScale(t,-1.0/(pep->sfactor*pep->sfactor));CHKERRQ(ierr);
+    } else {
+      ierr = STMatMult(pep->st,1,v,q);CHKERRQ(ierr);
+      ierr = STMatMult(pep->st,2,v,t);CHKERRQ(ierr);
+      ierr = VecAXPY(q,sigma*pep->sfactor,t);CHKERRQ(ierr);
+      ierr = VecScale(q,pep->sfactor);CHKERRQ(ierr);
+      ierr = BVMultVec(pep->V,1.0,0.0,v,S+offq+j*lds);CHKERRQ(ierr);
+      ierr = STMatMult(pep->st,2,v,t);CHKERRQ(ierr);
+      ierr = VecAXPY(q,pep->sfactor*pep->sfactor,t);CHKERRQ(ierr);
+      ierr = STMatSolve(pep->st,q,t);CHKERRQ(ierr);
+      ierr = VecScale(t,-1.0);CHKERRQ(ierr);
+    }
     ierr = BVRestoreColumn(pep->V,nqt,&t);CHKERRQ(ierr);
 
     /* orthogonalize */
-    ierr = BVOrthogonalizeColumn(pep->V,nqt,S+offq+(j+1)*lds,&norm,&lindep);CHKERRQ(ierr);
+    if (!sinvert) x = S+offq+(j+1)*lds;
+    else x = S+(j+1)*lds;
+    ierr = BVOrthogonalizeColumn(pep->V,nqt,x,&norm,&lindep);CHKERRQ(ierr);
+
     if (!lindep) {
-      *(S+offq+(j+1)*lds+nqt) = norm;
+      if (!sinvert) *(S+offq+(j+1)*lds+nqt) = norm;
+      else *(S+(j+1)*lds+nqt) = norm;
       ierr = BVScaleColumn(pep->V,nqt,1.0/norm);CHKERRQ(ierr);
       nqt++;
     }
-    for (i=0;i<=nqt-1;i++) *(S+(j+1)*lds+i) = *(S+offq+j*lds+i);
+    if (!sinvert) {
+      for (i=0;i<=nqt-1;i++) *(S+(j+1)*lds+i) = *(S+offq+j*lds+i);
+      if (ctx->beta && ctx->alpha) {
+        for (i=0;i<=nqt-1;i++) *(S+(j+1)*lds+offq+i) -= *(S+(j+1)*lds+i)*ctx->beta/ctx->alpha;
+      }
+    } else for (i=0;i<nqt;i++) *(S+(j+1)*lds+offq+i) = *(S+j*lds+i)+sigma*(*(S+(j+1)*lds+i));
     ierr = BVSetActiveColumns(pep->V,0,nqt);CHKERRQ(ierr);
     ierr = MatDenseRestoreArray(MS,&S);CHKERRQ(ierr);
     ierr = BVTensorRestoreFactors(ctx->V,NULL,&MS);CHKERRQ(ierr);
@@ -180,7 +274,7 @@ static PetscErrorCode PEPSTOARrun(PEP pep,PetscReal *a,PetscReal *b,PetscReal *o
     /* check symmetry */
     ierr = DSGetArrayReal(pep->ds,DS_MAT_T,&f);CHKERRQ(ierr);
     if (j==k) {
-      for (i=l;i<j-1;i++) y[i] = PetscAbsScalar(y[i])-PetscAbsReal(f[2*ld+i]);
+      for (i=l;i<j-1;i++) y[i] = PetscAbsScalar(y[i])-PetscAbsReal(f[2*ldds+i]);
       for (i=0;i<l;i++) y[i] = 0.0;
     }
     ierr = DSRestoreArrayReal(pep->ds,DS_MAT_T,&f);CHKERRQ(ierr);
@@ -205,7 +299,7 @@ static PetscErrorCode PEPSTOARrun(PEP pep,PetscReal *a,PetscReal *b,PetscReal *o
 static PetscErrorCode PEPSTOARpreKConvergence(PEP pep,PetscInt nv,PetscReal *norm,Vec *w)
 {
   PetscErrorCode ierr;
-  PEP_TOAR      *ctx = (PEP_TOAR*)pep->data;
+  PEP_STOAR      *ctx = (PEP_STOAR*)pep->data;
   PetscBLASInt   n_,one=1;
   PetscInt       lds=2*ctx->ld;
   PetscReal      t1,t2;
@@ -232,41 +326,41 @@ static PetscErrorCode PEPSTOARpreKConvergence(PEP pep,PetscInt nv,PetscReal *nor
 PetscErrorCode PEPSolve_STOAR(PEP pep)
 {
   PetscErrorCode ierr;
-  PEP_TOAR       *ctx = (PEP_TOAR*)pep->data;
-  PetscInt       j,k,l,nv=0,ld,ldds,t,nq=0,m,n;
+  PEP_STOAR      *ctx = (PEP_STOAR*)pep->data;
+  PetscInt       j,k,l,nv=0,ld,ldds,t,nq=0;
   PetscInt       nconv=0,deg=pep->nmat-1;
-  PetscScalar    *Q,*om,scal[2];
+  PetscScalar    *Q,*om,sigma;
   PetscReal      beta,norm=1.0,*omega,*a,*b,*r;
-  PetscBool      breakdown,symmlost=PETSC_FALSE,sinv,falselock=PETSC_TRUE;
-  Mat            MQ,A,pA[4],As[2],D[2];
+  PetscBool      breakdown,symmlost=PETSC_FALSE,sinv=PETSC_FALSE,falselock=PETSC_TRUE,flg;
+  Mat            MQ,A;
   Vec            vomega;
-  ShellMatCtx    *ctxMat[2];
 
   PetscFunctionBegin;
   ierr = PetscCitationsRegister(citation,&cited);CHKERRQ(ierr);
-  ierr = STGetMatrixTransformed(pep->st,2,&D[1]);CHKERRQ(ierr); /* M */
-  ierr = MatGetLocalSize(D[1],&m,&n);CHKERRQ(ierr);
-  ierr = STGetMatrixTransformed(pep->st,0,&D[0]);CHKERRQ(ierr); /* K */
-  scal[0] = -1.0; scal[1] = pep->sfactor*pep->sfactor;
-  for (j=0;j<2;j++) {
-    ierr = PetscNew(ctxMat+j);CHKERRQ(ierr);
-    (ctxMat[j])->A = D[j]; (ctxMat[j])->scal = scal[j];
-    ierr = MatCreateShell(PetscObjectComm((PetscObject)pep),m,n,PETSC_DETERMINE,PETSC_DETERMINE,ctxMat[j],&As[j]);CHKERRQ(ierr);
-    ierr = MatShellSetOperation(As[j],MATOP_MULT,(void(*)())MatMult_Func);CHKERRQ(ierr);
-    ierr = MatShellSetOperation(As[j],MATOP_DESTROY,(void(*)())MatDestroy_Func);CHKERRQ(ierr);
-  }
-  pA[0] = As[0]; pA[1] = pA[2] = NULL; pA[3] = As[1];
-  ierr = MatCreateNest(PetscObjectComm((PetscObject)pep),2,NULL,2,NULL,pA,&A);CHKERRQ(ierr);
-  for (j=0;j<2;j++) { ierr = MatDestroy(&As[j]);CHKERRQ(ierr); }
+  ierr = PEPSTOARSetUpInnerMatrix(pep,&A);CHKERRQ(ierr);
   ierr = BVSetMatrix(ctx->V,A,PETSC_TRUE);CHKERRQ(ierr);
   ierr = MatDestroy(&A);CHKERRQ(ierr);
   if (ctx->lock) {
+    /* undocumented option to use a cheaper locking instead of the true locking */
     ierr = PetscOptionsGetBool(NULL,NULL,"-pep_stoar_falselocking",&falselock,NULL);CHKERRQ(ierr);
   }
   ierr = BVGetSizes(pep->V,NULL,NULL,&ld);CHKERRQ(ierr);
-  ierr = PetscObjectTypeCompare((PetscObject)pep->st,STSINVERT,&sinv);CHKERRQ(ierr);
-  ierr = RGPushScale(pep->rg,sinv?pep->sfactor:1.0/pep->sfactor);CHKERRQ(ierr);
-  ierr = STScaleShift(pep->st,sinv?pep->sfactor:1.0/pep->sfactor);CHKERRQ(ierr);
+  ierr = STGetShift(pep->st,&sigma);CHKERRQ(ierr);
+  ierr = STGetTransform(pep->st,&flg);CHKERRQ(ierr);
+  if (pep->sfactor!=1.0) {
+    if (!flg) {
+      pep->target /= pep->sfactor;
+      ierr = RGPushScale(pep->rg,1.0/pep->sfactor);CHKERRQ(ierr);
+      ierr = STScaleShift(pep->st,1.0/pep->sfactor);CHKERRQ(ierr);
+      sigma /= pep->sfactor;
+    } else {
+      ierr = PetscObjectTypeCompare((PetscObject)pep->st,STSINVERT,&sinv);CHKERRQ(ierr);
+      pep->target = sinv?pep->target*pep->sfactor:pep->target/pep->sfactor;
+      ierr = RGPushScale(pep->rg,sinv?pep->sfactor:1.0/pep->sfactor);CHKERRQ(ierr);
+      ierr = STScaleShift(pep->st,sinv?pep->sfactor:1.0/pep->sfactor);CHKERRQ(ierr);
+    }
+  }
+  if (flg) sigma = 0.0;
 
   /* Get the starting Arnoldi vector */
   ierr = BVTensorBuildFirstColumn(ctx->V,pep->nini);CHKERRQ(ierr);
@@ -374,7 +468,7 @@ PetscErrorCode PEPSolve_STOAR(PEP pep)
       ierr = PetscInfo2(pep,"Breakdown TOAR method (it=%D norm=%g)\n",pep->its,(double)beta);CHKERRQ(ierr);
       pep->reason = PEP_DIVERGED_BREAKDOWN;
     }
-    if (pep->reason != PEP_CONVERGED_ITERATING) l--; 
+    if (pep->reason != PEP_CONVERGED_ITERATING) l--;
     ierr = BVGetActiveColumns(pep->V,NULL,&nq);CHKERRQ(ierr);
     if (k+l+deg<=nq) {
       ierr = BVSetActiveColumns(ctx->V,pep->nconv,k+l+1);CHKERRQ(ierr);
@@ -396,13 +490,26 @@ PetscErrorCode PEPSolve_STOAR(PEP pep)
       ierr = BVTensorCompress(ctx->V,pep->nconv);CHKERRQ(ierr);
       ierr = BVSetActiveColumns(pep->V,0,pep->nconv);CHKERRQ(ierr);
     }
+  }
+  ierr = STGetTransform(pep->st,&flg);CHKERRQ(ierr);
+  if (!flg && pep->ops->backtransform) {
+      ierr = (*pep->ops->backtransform)(pep);CHKERRQ(ierr);
+  }
+  if (pep->sfactor!=1.0) {
     for (j=0;j<pep->nconv;j++) {
       pep->eigr[j] *= pep->sfactor;
       pep->eigi[j] *= pep->sfactor;
     }
   }
-  ierr = STScaleShift(pep->st,sinv?1.0/pep->sfactor:pep->sfactor);CHKERRQ(ierr);
-  ierr = RGPopScale(pep->rg);CHKERRQ(ierr);
+  /* restore original values */
+  if (!flg) {
+    pep->target *= pep->sfactor;
+    ierr = STScaleShift(pep->st,pep->sfactor);CHKERRQ(ierr);
+  } else {
+    ierr = STScaleShift(pep->st,sinv?1.0/pep->sfactor:pep->sfactor);CHKERRQ(ierr);
+    pep->target = (sinv)?pep->target/pep->sfactor:pep->target*pep->sfactor;
+  }
+  if (pep->sfactor!=1.0) { ierr = RGPopScale(pep->rg);CHKERRQ(ierr); }
 
   /* truncate Schur decomposition and change the state to raw so that
      DSVectors() computes eigenvectors from scratch */
@@ -416,24 +523,35 @@ PetscErrorCode PEPSetFromOptions_STOAR(PetscOptionItems *PetscOptionsObject,PEP
   PetscErrorCode ierr;
   PetscBool      flg,lock,b,f1,f2,f3;
   PetscInt       i,j,k;
-  PEP_TOAR *ctx = (PEP_TOAR*)pep->data;
+  PetscReal      array[2]={0,0};
+  PEP_STOAR      *ctx = (PEP_STOAR*)pep->data;
 
   PetscFunctionBegin;
   ierr = PetscOptionsHead(PetscOptionsObject,"PEP STOAR Options");CHKERRQ(ierr);
 
-  ierr = PetscOptionsBool("-pep_stoar_locking","Choose between locking and non-locking variants","PEPSTOARSetLocking",PETSC_FALSE,&lock,&flg);CHKERRQ(ierr);
-  if (flg) { ierr = PEPSTOARSetLocking(pep,lock);CHKERRQ(ierr); }
+    ierr = PetscOptionsBool("-pep_stoar_locking","Choose between locking and non-locking variants","PEPSTOARSetLocking",PETSC_FALSE,&lock,&flg);CHKERRQ(ierr);
+    if (flg) { ierr = PEPSTOARSetLocking(pep,lock);CHKERRQ(ierr); }
 
-  b = ctx->detect;
-  ierr = PetscOptionsBool("-pep_stoar_detect_zeros","Check zeros during factorizations at interval boundaries","PEPSTOARSetDetectZeros",ctx->detect,&b,&flg);CHKERRQ(ierr);
-  if (flg) { ierr = PEPSTOARSetDetectZeros(pep,b);CHKERRQ(ierr); }
+    b = ctx->detect;
+    ierr = PetscOptionsBool("-pep_stoar_detect_zeros","Check zeros during factorizations at interval boundaries","PEPSTOARSetDetectZeros",ctx->detect,&b,&flg);CHKERRQ(ierr);
+    if (flg) { ierr = PEPSTOARSetDetectZeros(pep,b);CHKERRQ(ierr); }
 
-  i = 1;
-  j = k = PETSC_DECIDE;
-  ierr = PetscOptionsInt("-pep_stoar_nev","Number of eigenvalues to compute in each subsolve (only for spectrum slicing)","PEPSTOARSetDimensions",20,&i,&f1);CHKERRQ(ierr);
-  ierr = PetscOptionsInt("-pep_stoar_ncv","Number of basis vectors in each subsolve (only for spectrum slicing)","PEPSTOARSetDimensions",40,&j,&f2);CHKERRQ(ierr);
-  ierr = PetscOptionsInt("-pep_stoar_mpd","Maximum dimension of projected problem in each subsolve (only for spectrum slicing)","PEPSTOARSetDimensions",40,&k,&f3);CHKERRQ(ierr);
-  if (f1 || f2 || f3) { ierr = PEPSTOARSetDimensions(pep,i,j,k);CHKERRQ(ierr); }
+    i = 1;
+    j = k = PETSC_DECIDE;
+    ierr = PetscOptionsInt("-pep_stoar_nev","Number of eigenvalues to compute in each subsolve (only for spectrum slicing)","PEPSTOARSetDimensions",20,&i,&f1);CHKERRQ(ierr);
+    ierr = PetscOptionsInt("-pep_stoar_ncv","Number of basis vectors in each subsolve (only for spectrum slicing)","PEPSTOARSetDimensions",40,&j,&f2);CHKERRQ(ierr);
+    ierr = PetscOptionsInt("-pep_stoar_mpd","Maximum dimension of projected problem in each subsolve (only for spectrum slicing)","PEPSTOARSetDimensions",40,&k,&f3);CHKERRQ(ierr);
+    if (f1 || f2 || f3) { ierr = PEPSTOARSetDimensions(pep,i,j,k);CHKERRQ(ierr); }
+
+    k = 2;
+    ierr = PetscOptionsRealArray("-pep_stoar_linearization","Parameters of the linearization","PEPSTOARSetLinearization",array,&k,&flg);CHKERRQ(ierr);
+    if (flg) {
+      ierr = PEPSTOARSetLinearization(pep,array[0],array[1]);CHKERRQ(ierr);
+    }
+
+    b = ctx->checket;
+    ierr = PetscOptionsBool("-pep_stoar_check_eigenvalue_type","Check eigenvalue type during spectrum slicing","PEPSTOARSetCheckEigenvalueType",ctx->checket,&b,&flg);CHKERRQ(ierr);
+    if (flg) { ierr = PEPSTOARSetCheckEigenvalueType(pep,b);CHKERRQ(ierr); }
 
   ierr = PetscOptionsTail();CHKERRQ(ierr);
   PetscFunctionReturn(0);
@@ -441,7 +559,7 @@ PetscErrorCode PEPSetFromOptions_STOAR(PetscOptionItems *PetscOptionsObject,PEP
 
 static PetscErrorCode PEPSTOARSetLocking_STOAR(PEP pep,PetscBool lock)
 {
-  PEP_TOAR *ctx = (PEP_TOAR*)pep->data;
+  PEP_STOAR *ctx = (PEP_STOAR*)pep->data;
 
   PetscFunctionBegin;
   ctx->lock = lock;
@@ -452,7 +570,7 @@ static PetscErrorCode PEPSTOARSetLocking_STOAR(PEP pep,PetscBool lock)
    PEPSTOARSetLocking - Choose between locking and non-locking variants of
    the STOAR method.
 
-   Logically Collective on PEP
+   Logically Collective on pep
 
    Input Parameters:
 +  pep  - the eigenproblem solver context
@@ -484,7 +602,7 @@ PetscErrorCode PEPSTOARSetLocking(PEP pep,PetscBool lock)
 
 static PetscErrorCode PEPSTOARGetLocking_STOAR(PEP pep,PetscBool *lock)
 {
-  PEP_TOAR *ctx = (PEP_TOAR*)pep->data;
+  PEP_STOAR *ctx = (PEP_STOAR*)pep->data;
 
   PetscFunctionBegin;
   *lock = ctx->lock;
@@ -512,7 +630,7 @@ PetscErrorCode PEPSTOARGetLocking(PEP pep,PetscBool *lock)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(pep,PEP_CLASSID,1);
-  PetscValidPointer(lock,2);
+  PetscValidBoolPointer(lock,2);
   ierr = PetscUseMethod(pep,"PEPSTOARGetLocking_C",(PEP,PetscBool*),(pep,lock));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -521,7 +639,7 @@ static PetscErrorCode PEPSTOARGetInertias_STOAR(PEP pep,PetscInt *n,PetscReal **
 {
   PetscErrorCode ierr;
   PetscInt       i,numsh;
-  PEP_TOAR       *ctx = (PEP_TOAR*)pep->data;
+  PEP_STOAR      *ctx = (PEP_STOAR*)pep->data;
   PEP_SR         sr = ctx->sr;
 
   PetscFunctionBegin;
@@ -617,7 +735,7 @@ PetscErrorCode PEPSTOARGetInertias(PEP pep,PetscInt *n,PetscReal **shifts,PetscI
 
 static PetscErrorCode PEPSTOARSetDetectZeros_STOAR(PEP pep,PetscBool detect)
 {
-  PEP_TOAR *ctx = (PEP_TOAR*)pep->data;
+  PEP_STOAR *ctx = (PEP_STOAR*)pep->data;
 
   PetscFunctionBegin;
   ctx->detect = detect;
@@ -629,7 +747,7 @@ static PetscErrorCode PEPSTOARSetDetectZeros_STOAR(PEP pep,PetscBool detect)
    PEPSTOARSetDetectZeros - Sets a flag to enforce detection of
    zeros during the factorizations throughout the spectrum slicing computation.
 
-   Logically Collective on PEP
+   Logically Collective on pep
 
    Input Parameters:
 +  pep    - the eigenproblem solver context
@@ -648,7 +766,7 @@ static PetscErrorCode PEPSTOARSetDetectZeros_STOAR(PEP pep,PetscBool detect)
 
    Level: advanced
 
-.seealso: PEPSTOARSetPartitions(), PEPSetInterval()
+.seealso: PEPSetInterval()
 @*/
 PetscErrorCode PEPSTOARSetDetectZeros(PEP pep,PetscBool detect)
 {
@@ -663,7 +781,7 @@ PetscErrorCode PEPSTOARSetDetectZeros(PEP pep,PetscBool detect)
 
 static PetscErrorCode PEPSTOARGetDetectZeros_STOAR(PEP pep,PetscBool *detect)
 {
-  PEP_TOAR *ctx = (PEP_TOAR*)pep->data;
+  PEP_STOAR *ctx = (PEP_STOAR*)pep->data;
 
   PetscFunctionBegin;
   *detect = ctx->detect;
@@ -692,14 +810,96 @@ PetscErrorCode PEPSTOARGetDetectZeros(PEP pep,PetscBool *detect)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(pep,PEP_CLASSID,1);
-  PetscValidPointer(detect,2);
+  PetscValidBoolPointer(detect,2);
   ierr = PetscUseMethod(pep,"PEPSTOARGetDetectZeros_C",(PEP,PetscBool*),(pep,detect));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
+static PetscErrorCode PEPSTOARSetLinearization_STOAR(PEP pep,PetscReal alpha,PetscReal beta)
+{
+  PEP_STOAR *ctx = (PEP_STOAR*)pep->data;
+
+  PetscFunctionBegin;
+  if (beta==0.0 && alpha==0.0) SETERRQ(PetscObjectComm((PetscObject)pep),PETSC_ERR_ARG_WRONG,"Parameters alpha and beta cannot be zero simultaneously");
+  ctx->alpha = alpha;
+  ctx->beta  = beta;
+  PetscFunctionReturn(0);
+}
+
+/*@
+   PEPSTOARSetLinearization - Set the coefficients that define
+   the linearization of a quadratic eigenproblem.
+
+   Logically Collective on pep
+
+   Input Parameters:
++  pep   - polynomial eigenvalue solver
+.  alpha - first parameter of the linearization
+-  beta  - second parameter of the linearization
+
+   Options Database Key:
+.  -pep_stoar_linearization <alpha,beta> - Sets the coefficients
+
+   Notes:
+   Cannot pass zero for both alpha and beta. The default values are
+   alpha=1 and beta=0.
+
+   Level: advanced
+
+.seealso: PEPSTOARGetLinearization()
+@*/
+PetscErrorCode PEPSTOARSetLinearization(PEP pep,PetscReal alpha,PetscReal beta)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(pep,PEP_CLASSID,1);
+  PetscValidLogicalCollectiveReal(pep,alpha,2);
+  PetscValidLogicalCollectiveReal(pep,beta,3);
+  ierr = PetscTryMethod(pep,"PEPSTOARSetLinearization_C",(PEP,PetscReal,PetscReal),(pep,alpha,beta));CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+static PetscErrorCode PEPSTOARGetLinearization_STOAR(PEP pep,PetscReal *alpha,PetscReal *beta)
+{
+  PEP_STOAR *ctx = (PEP_STOAR*)pep->data;
+
+  PetscFunctionBegin;
+  if (alpha) *alpha = ctx->alpha;
+  if (beta)  *beta  = ctx->beta;
+  PetscFunctionReturn(0);
+}
+
+/*@
+   PEPSTOARGetLinearization - Returns the coefficients that define
+   the linearization of a quadratic eigenproblem.
+
+   Not Collective
+
+   Input Parameter:
+.  pep  - polynomial eigenvalue solver
+
+   Output Parameters:
++  alpha - the first parameter of the linearization
+-  beta  - the second parameter of the linearization
+
+   Level: advanced
+
+.seealso: PEPSTOARSetLinearization()
+@*/
+PetscErrorCode PEPSTOARGetLinearization(PEP pep,PetscReal *alpha,PetscReal *beta)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(pep,PEP_CLASSID,1);
+  ierr = PetscUseMethod(pep,"PEPSTOARGetLinearization_C",(PEP,PetscReal*,PetscReal*),(pep,alpha,beta));CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
 static PetscErrorCode PEPSTOARSetDimensions_STOAR(PEP pep,PetscInt nev,PetscInt ncv,PetscInt mpd)
 {
-  PEP_TOAR *ctx = (PEP_TOAR*)pep->data;
+  PEP_STOAR *ctx = (PEP_STOAR*)pep->data;
 
   PetscFunctionBegin;
   if (nev<1) SETERRQ(PetscObjectComm((PetscObject)pep),PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of nev. Must be > 0");
@@ -725,7 +925,7 @@ static PetscErrorCode PEPSTOARSetDimensions_STOAR(PEP pep,PetscInt nev,PetscInt
    step in case of doing spectrum slicing for a computational interval.
    The meaning of the parameters is the same as in PEPSetDimensions().
 
-   Logically Collective on PEP
+   Logically Collective on pep
 
    Input Parameters:
 +  pep - the eigenproblem solver context
@@ -757,7 +957,7 @@ PetscErrorCode PEPSTOARSetDimensions(PEP pep,PetscInt nev,PetscInt ncv,PetscInt
 
 static PetscErrorCode PEPSTOARGetDimensions_STOAR(PEP pep,PetscInt *nev,PetscInt *ncv,PetscInt *mpd)
 {
-  PEP_TOAR *ctx = (PEP_TOAR*)pep->data;
+  PEP_STOAR *ctx = (PEP_STOAR*)pep->data;
 
   PetscFunctionBegin;
   if (nev) *nev = ctx->nev;
@@ -794,16 +994,105 @@ PetscErrorCode PEPSTOARGetDimensions(PEP pep,PetscInt *nev,PetscInt *ncv,PetscIn
   PetscFunctionReturn(0);
 }
 
+static PetscErrorCode PEPSTOARSetCheckEigenvalueType_STOAR(PEP pep,PetscBool checket)
+{
+  PEP_STOAR *ctx = (PEP_STOAR*)pep->data;
+
+  PetscFunctionBegin;
+  ctx->checket = checket;
+  pep->state   = PEP_STATE_INITIAL;
+  PetscFunctionReturn(0);
+}
+
+/*@
+   PEPSTOARSetCheckEigenvalueType - Sets a flag to check that all the eigenvalues
+   obtained throughout the spectrum slicing computation have the same definite type.
+
+   Logically Collective on pep
+
+   Input Parameters:
++  pep     - the eigenproblem solver context
+-  checket - check eigenvalue type
+
+   Options Database Key:
+.  -pep_stoar_check_eigenvalue_type - Check eigenvalue type; this takes an optional
+   bool value (0/1/no/yes/true/false)
+
+   Notes:
+   This option is relevant only for spectrum slicing computations, but it is
+   ignored if the problem type is PEP_HYPERBOLIC.
+
+   This flag is turned on by default, to guarantee that the computed eigenvalues
+   have the same type (otherwise the computed solution might be wrong). But since
+   the check is computationally quite expensive, the check may be turned off if
+   the user knows for sure that all eigenvalues in the requested interval have
+   the same type.
+
+   Level: advanced
+
+.seealso: PEPSetProblemType(), PEPSetInterval()
+@*/
+PetscErrorCode PEPSTOARSetCheckEigenvalueType(PEP pep,PetscBool checket)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(pep,PEP_CLASSID,1);
+  PetscValidLogicalCollectiveBool(pep,checket,2);
+  ierr = PetscTryMethod(pep,"PEPSTOARSetCheckEigenvalueType_C",(PEP,PetscBool),(pep,checket));CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+static PetscErrorCode PEPSTOARGetCheckEigenvalueType_STOAR(PEP pep,PetscBool *checket)
+{
+  PEP_STOAR *ctx = (PEP_STOAR*)pep->data;
+
+  PetscFunctionBegin;
+  *checket = ctx->checket;
+  PetscFunctionReturn(0);
+}
+
+/*@
+   PEPSTOARGetCheckEigenvalueType - Gets the flag for the eigenvalue type
+   check in spectrum slicing.
+
+   Not Collective
+
+   Input Parameter:
+.  pep - the eigenproblem solver context
+
+   Output Parameter:
+.  checket - whether eigenvalue type must be checked during spectrum slcing
+
+   Level: advanced
+
+.seealso: PEPSTOARSetCheckEigenvalueType()
+@*/
+PetscErrorCode PEPSTOARGetCheckEigenvalueType(PEP pep,PetscBool *checket)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(pep,PEP_CLASSID,1);
+  PetscValidBoolPointer(checket,2);
+  ierr = PetscUseMethod(pep,"PEPSTOARGetCheckEigenvalueType_C",(PEP,PetscBool*),(pep,checket));CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
 PetscErrorCode PEPView_STOAR(PEP pep,PetscViewer viewer)
 {
   PetscErrorCode ierr;
-  PEP_TOAR      *ctx = (PEP_TOAR*)pep->data;
+  PEP_STOAR      *ctx = (PEP_STOAR*)pep->data;
   PetscBool      isascii;
 
   PetscFunctionBegin;
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
   if (isascii) {
     ierr = PetscViewerASCIIPrintf(viewer,"  using the %slocking variant\n",ctx->lock?"":"non-");CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  linearization parameters: alpha=%g beta=%g\n",(double)ctx->alpha,(double)ctx->beta);CHKERRQ(ierr);
+    if (pep->which==PEP_ALL && !ctx->hyperbolic) {
+      ierr = PetscViewerASCIIPrintf(viewer,"  checking eigenvalue type: %s\n",ctx->checket?"enabled":"disabled");CHKERRQ(ierr);
+    }
   }
   PetscFunctionReturn(0);
 }
@@ -822,7 +1111,7 @@ PetscErrorCode PEPReset_STOAR(PEP pep)
 PetscErrorCode PEPDestroy_STOAR(PEP pep)
 {
   PetscErrorCode ierr;
-  PEP_TOAR       *ctx = (PEP_TOAR*)pep->data;
+  PEP_STOAR      *ctx = (PEP_STOAR*)pep->data;
 
   PetscFunctionBegin;
   ierr = BVDestroy(&ctx->V);CHKERRQ(ierr);
@@ -834,18 +1123,26 @@ PetscErrorCode PEPDestroy_STOAR(PEP pep)
   ierr = PetscObjectComposeFunction((PetscObject)pep,"PEPSTOARGetInertias_C",NULL);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)pep,"PEPSTOARGetDimensions_C",NULL);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)pep,"PEPSTOARSetDimensions_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)pep,"PEPSTOARSetLinearization_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)pep,"PEPSTOARGetLinearization_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)pep,"PEPSTOARSetCheckEigenvalueType_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)pep,"PEPSTOARGetCheckEigenvalueType_C",NULL);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-PETSC_EXTERN PetscErrorCode PEPCreate_STOAR(PEP pep)
+SLEPC_EXTERN PetscErrorCode PEPCreate_STOAR(PEP pep)
 {
   PetscErrorCode ierr;
-  PEP_TOAR      *ctx;
+  PEP_STOAR      *ctx;
 
   PetscFunctionBegin;
   ierr = PetscNewLog(pep,&ctx);CHKERRQ(ierr);
   pep->data = (void*)ctx;
-  ctx->lock = PETSC_TRUE;
+  ctx->lock    = PETSC_TRUE;
+  ctx->nev     = 1;
+  ctx->alpha   = 1.0;
+  ctx->beta    = 0.0;
+  ctx->checket = PETSC_TRUE;
 
   pep->ops->setup          = PEPSetUp_STOAR;
   pep->ops->setfromoptions = PEPSetFromOptions_STOAR;
@@ -854,7 +1151,6 @@ PETSC_EXTERN PetscErrorCode PEPCreate_STOAR(PEP pep)
   pep->ops->backtransform  = PEPBackTransform_Default;
   pep->ops->computevectors = PEPComputeVectors_Default;
   pep->ops->extractvectors = PEPExtractVectors_TOAR;
-  pep->ops->setdefaultst   = PEPSetDefaultST_Transform;
   pep->ops->reset          = PEPReset_STOAR;
 
   ierr = PetscObjectComposeFunction((PetscObject)pep,"PEPSTOARSetLocking_C",PEPSTOARSetLocking_STOAR);CHKERRQ(ierr);
@@ -864,6 +1160,10 @@ PETSC_EXTERN PetscErrorCode PEPCreate_STOAR(PEP pep)
   ierr = PetscObjectComposeFunction((PetscObject)pep,"PEPSTOARGetInertias_C",PEPSTOARGetInertias_STOAR);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)pep,"PEPSTOARGetDimensions_C",PEPSTOARGetDimensions_STOAR);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)pep,"PEPSTOARSetDimensions_C",PEPSTOARSetDimensions_STOAR);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)pep,"PEPSTOARSetLinearization_C",PEPSTOARSetLinearization_STOAR);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)pep,"PEPSTOARGetLinearization_C",PEPSTOARGetLinearization_STOAR);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)pep,"PEPSTOARSetCheckEigenvalueType_C",PEPSTOARSetCheckEigenvalueType_STOAR);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)pep,"PEPSTOARGetCheckEigenvalueType_C",PEPSTOARGetCheckEigenvalueType_STOAR);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
