@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2018, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -21,7 +21,7 @@ PetscLogEvent     PEP_SetUp = 0,PEP_Solve = 0,PEP_Refine = 0;
 /*@
    PEPCreate - Creates the default PEP context.
 
-   Collective on MPI_Comm
+   Collective
 
    Input Parameter:
 .  comm - MPI communicator
@@ -121,7 +121,7 @@ PetscErrorCode PEPCreate(MPI_Comm comm,PEP *outpep)
 /*@C
    PEPSetType - Selects the particular solver to be used in the PEP object.
 
-   Logically Collective on PEP
+   Logically Collective on pep
 
    Input Parameters:
 +  pep      - the polynomial eigensolver context
@@ -226,6 +226,7 @@ PetscErrorCode PEPRegister(const char *name,PetscErrorCode (*function)(PEP))
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
+  ierr = PEPInitializePackage();CHKERRQ(ierr);
   ierr = PetscFunctionListAdd(&PEPList,name,function);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -234,7 +235,7 @@ PetscErrorCode PEPRegister(const char *name,PetscErrorCode (*function)(PEP))
    PEPReset - Resets the PEP context to the initial state (prior to setup)
    and destroys any allocated Vecs and Mats.
 
-   Collective on PEP
+   Collective on pep
 
    Input Parameter:
 .  pep - eigensolver context obtained from PEPCreate()
@@ -271,7 +272,7 @@ PetscErrorCode PEPReset(PEP pep)
 /*@
    PEPDestroy - Destroys the PEP context.
 
-   Collective on PEP
+   Collective on pep
 
    Input Parameter:
 .  pep - eigensolver context obtained from PEPCreate()
@@ -312,7 +313,7 @@ PetscErrorCode PEPDestroy(PEP *pep)
 /*@
    PEPSetBV - Associates a basis vectors object to the polynomial eigensolver.
 
-   Collective on PEP
+   Collective on pep
 
    Input Parameters:
 +  pep - eigensolver context obtained from PEPCreate()
@@ -366,7 +367,9 @@ PetscErrorCode PEPGetBV(PEP pep,BV *bv)
   PetscValidPointer(bv,2);
   if (!pep->V) {
     ierr = BVCreate(PetscObjectComm((PetscObject)pep),&pep->V);CHKERRQ(ierr);
+    ierr = PetscObjectIncrementTabLevel((PetscObject)pep->V,(PetscObject)pep,0);CHKERRQ(ierr);
     ierr = PetscLogObjectParent((PetscObject)pep,(PetscObject)pep->V);CHKERRQ(ierr);
+    ierr = PetscObjectSetOptions((PetscObject)pep->V,((PetscObject)pep)->options);CHKERRQ(ierr);
   }
   *bv = pep->V;
   PetscFunctionReturn(0);
@@ -375,7 +378,7 @@ PetscErrorCode PEPGetBV(PEP pep,BV *bv)
 /*@
    PEPSetRG - Associates a region object to the polynomial eigensolver.
 
-   Collective on PEP
+   Collective on pep
 
    Input Parameters:
 +  pep - eigensolver context obtained from PEPCreate()
@@ -429,7 +432,9 @@ PetscErrorCode PEPGetRG(PEP pep,RG *rg)
   PetscValidPointer(rg,2);
   if (!pep->rg) {
     ierr = RGCreate(PetscObjectComm((PetscObject)pep),&pep->rg);CHKERRQ(ierr);
+    ierr = PetscObjectIncrementTabLevel((PetscObject)pep->rg,(PetscObject)pep,0);CHKERRQ(ierr);
     ierr = PetscLogObjectParent((PetscObject)pep,(PetscObject)pep->rg);CHKERRQ(ierr);
+    ierr = PetscObjectSetOptions((PetscObject)pep->rg,((PetscObject)pep)->options);CHKERRQ(ierr);
   }
   *rg = pep->rg;
   PetscFunctionReturn(0);
@@ -438,7 +443,7 @@ PetscErrorCode PEPGetRG(PEP pep,RG *rg)
 /*@
    PEPSetDS - Associates a direct solver object to the polynomial eigensolver.
 
-   Collective on PEP
+   Collective on pep
 
    Input Parameters:
 +  pep - eigensolver context obtained from PEPCreate()
@@ -492,7 +497,9 @@ PetscErrorCode PEPGetDS(PEP pep,DS *ds)
   PetscValidPointer(ds,2);
   if (!pep->ds) {
     ierr = DSCreate(PetscObjectComm((PetscObject)pep),&pep->ds);CHKERRQ(ierr);
+    ierr = PetscObjectIncrementTabLevel((PetscObject)pep->ds,(PetscObject)pep,0);CHKERRQ(ierr);
     ierr = PetscLogObjectParent((PetscObject)pep,(PetscObject)pep->ds);CHKERRQ(ierr);
+    ierr = PetscObjectSetOptions((PetscObject)pep->ds,((PetscObject)pep)->options);CHKERRQ(ierr);
   }
   *ds = pep->ds;
   PetscFunctionReturn(0);
@@ -501,7 +508,7 @@ PetscErrorCode PEPGetDS(PEP pep,DS *ds)
 /*@
    PEPSetST - Associates a spectral transformation object to the eigensolver.
 
-   Collective on PEP
+   Collective on pep
 
    Input Parameters:
 +  pep - eigensolver context obtained from PEPCreate()
@@ -555,7 +562,9 @@ PetscErrorCode PEPGetST(PEP pep,ST *st)
   PetscValidPointer(st,2);
   if (!pep->st) {
     ierr = STCreate(PetscObjectComm((PetscObject)pep),&pep->st);CHKERRQ(ierr);
+    ierr = PetscObjectIncrementTabLevel((PetscObject)pep->st,(PetscObject)pep,0);CHKERRQ(ierr);
     ierr = PetscLogObjectParent((PetscObject)pep,(PetscObject)pep->st);CHKERRQ(ierr);
+    ierr = PetscObjectSetOptions((PetscObject)pep->st,((PetscObject)pep)->options);CHKERRQ(ierr);
   }
   *st = pep->st;
   PetscFunctionReturn(0);
@@ -593,7 +602,9 @@ PetscErrorCode PEPRefineGetKSP(PEP pep,KSP *ksp)
       ierr = PetscLogObjectMemory((PetscObject)pep,sizeof(PetscSubcomm));CHKERRQ(ierr);
     }
     ierr = KSPCreate((pep->npart==1)?PetscObjectComm((PetscObject)pep):PetscSubcommChild(pep->refinesubc),&pep->refineksp);CHKERRQ(ierr);
+    ierr = PetscObjectIncrementTabLevel((PetscObject)pep->refineksp,(PetscObject)pep,0);CHKERRQ(ierr);
     ierr = PetscLogObjectParent((PetscObject)pep,(PetscObject)pep->refineksp);CHKERRQ(ierr);
+    ierr = PetscObjectSetOptions((PetscObject)pep->refineksp,((PetscObject)pep)->options);CHKERRQ(ierr);
     ierr = KSPSetOptionsPrefix(*ksp,((PetscObject)pep)->prefix);CHKERRQ(ierr);
     ierr = KSPAppendOptionsPrefix(*ksp,"pep_refine_");CHKERRQ(ierr);
     ierr = KSPSetTolerances(pep->refineksp,SLEPC_DEFAULT_TOL,PETSC_DEFAULT,PETSC_DEFAULT,PETSC_DEFAULT);CHKERRQ(ierr);
@@ -605,7 +616,7 @@ PetscErrorCode PEPRefineGetKSP(PEP pep,KSP *ksp)
 /*@
    PEPSetTarget - Sets the value of the target.
 
-   Logically Collective on PEP
+   Logically Collective on pep
 
    Input Parameters:
 +  pep    - eigensolver context
@@ -669,7 +680,7 @@ PetscErrorCode PEPGetTarget(PEP pep,PetscScalar* target)
 /*@
    PEPSetInterval - Defines the computational interval for spectrum slicing.
 
-   Logically Collective on PEP
+   Logically Collective on pep
 
    Input Parameters:
 +  pep  - eigensolver context
