@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -114,18 +114,24 @@ PetscErrorCode SVDSetUp_Cross(SVD svd)
   if (!cross->mat) {
     if (cross->explicitmatrix) {
       if (svd->A && svd->AT) {  /* explicit transpose */
-        ierr = MatMatMult(svd->AT,svd->A,MAT_INITIAL_MATRIX,PETSC_DEFAULT,&cross->mat);CHKERRQ(ierr);
+        ierr = MatProductCreate(svd->AT,svd->A,NULL,&cross->mat);CHKERRQ(ierr);
+        ierr = MatProductSetType(cross->mat,MATPRODUCT_AB);CHKERRQ(ierr);
       } else {  /* implicit transpose */
 #if defined(PETSC_USE_COMPLEX)
         SETERRQ(PetscObjectComm((PetscObject)svd),PETSC_ERR_SUP,"Must use explicit transpose with complex scalars");
 #else
         if (svd->A) {
-          ierr = MatTransposeMatMult(svd->A,svd->A,MAT_INITIAL_MATRIX,PETSC_DEFAULT,&cross->mat);CHKERRQ(ierr);
+          ierr = MatProductCreate(svd->A,svd->A,NULL,&cross->mat);CHKERRQ(ierr);
+          ierr = MatProductSetType(cross->mat,MATPRODUCT_AtB);CHKERRQ(ierr);
         } else {
-          ierr = MatMatTransposeMult(svd->AT,svd->AT,MAT_INITIAL_MATRIX,PETSC_DEFAULT,&cross->mat);CHKERRQ(ierr);
+          ierr = MatProductCreate(svd->AT,svd->AT,NULL,&cross->mat);CHKERRQ(ierr);
+          ierr = MatProductSetType(cross->mat,MATPRODUCT_ABt);CHKERRQ(ierr);
         }
 #endif
       }
+      ierr = MatProductSetFromOptions(cross->mat);CHKERRQ(ierr);
+      ierr = MatProductSymbolic(cross->mat);CHKERRQ(ierr);
+      ierr = MatProductNumeric(cross->mat);CHKERRQ(ierr);
     } else {
       ierr = SVDMatGetLocalSize(svd,NULL,&n);CHKERRQ(ierr);
       ierr = MatCreateShell(PetscObjectComm((PetscObject)svd),n,n,PETSC_DETERMINE,PETSC_DETERMINE,svd,&cross->mat);CHKERRQ(ierr);
@@ -261,7 +267,7 @@ static PetscErrorCode SVDCrossSetExplicitMatrix_Cross(SVD svd,PetscBool explicit
    SVDCrossSetExplicitMatrix - Indicate if the eigensolver operator A^T*A must
    be computed explicitly.
 
-   Logically Collective on SVD
+   Logically Collective on svd
 
    Input Parameters:
 +  svd      - singular value solver
@@ -315,7 +321,7 @@ PetscErrorCode SVDCrossGetExplicitMatrix(SVD svd,PetscBool *explicitmatrix)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
-  PetscValidPointer(explicitmatrix,2);
+  PetscValidBoolPointer(explicitmatrix,2);
   ierr = PetscUseMethod(svd,"SVDCrossGetExplicitMatrix_C",(SVD,PetscBool*),(svd,explicitmatrix));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -339,7 +345,7 @@ static PetscErrorCode SVDCrossSetEPS_Cross(SVD svd,EPS eps)
    SVDCrossSetEPS - Associate an eigensolver object (EPS) to the
    singular value solver.
 
-   Collective on SVD
+   Collective on svd
 
    Input Parameters:
 +  svd - singular value solver
@@ -369,10 +375,11 @@ static PetscErrorCode SVDCrossGetEPS_Cross(SVD svd,EPS *eps)
   PetscFunctionBegin;
   if (!cross->eps) {
     ierr = EPSCreate(PetscObjectComm((PetscObject)svd),&cross->eps);CHKERRQ(ierr);
+    ierr = PetscObjectIncrementTabLevel((PetscObject)cross->eps,(PetscObject)svd,1);CHKERRQ(ierr);
     ierr = EPSSetOptionsPrefix(cross->eps,((PetscObject)svd)->prefix);CHKERRQ(ierr);
     ierr = EPSAppendOptionsPrefix(cross->eps,"svd_cross_");CHKERRQ(ierr);
-    ierr = PetscObjectIncrementTabLevel((PetscObject)cross->eps,(PetscObject)svd,1);CHKERRQ(ierr);
     ierr = PetscLogObjectParent((PetscObject)svd,(PetscObject)cross->eps);CHKERRQ(ierr);
+    ierr = PetscObjectSetOptions((PetscObject)cross->eps,((PetscObject)svd)->options);CHKERRQ(ierr);
     ierr = EPSSetWhichEigenpairs(cross->eps,EPS_LARGEST_REAL);CHKERRQ(ierr);
     ierr = EPSMonitorSet(cross->eps,EPSMonitor_Cross,svd,NULL);CHKERRQ(ierr);
   }
@@ -453,7 +460,7 @@ PetscErrorCode SVDDestroy_Cross(SVD svd)
   PetscFunctionReturn(0);
 }
 
-PETSC_EXTERN PetscErrorCode SVDCreate_Cross(SVD svd)
+SLEPC_EXTERN PetscErrorCode SVDCreate_Cross(SVD svd)
 {
   PetscErrorCode ierr;
   SVD_CROSS      *cross;
