@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -18,7 +18,7 @@
    SVDSetImplicitTranspose - Indicates how to handle the transpose of the matrix
    associated with the singular value problem.
 
-   Logically Collective on SVD
+   Logically Collective on svd
 
    Input Parameters:
 +  svd  - the singular value solver context
@@ -73,7 +73,7 @@ PetscErrorCode SVDGetImplicitTranspose(SVD svd,PetscBool *impl)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
-  PetscValidPointer(impl,2);
+  PetscValidBoolPointer(impl,2);
   *impl = svd->impltrans;
   PetscFunctionReturn(0);
 }
@@ -82,7 +82,7 @@ PetscErrorCode SVDGetImplicitTranspose(SVD svd,PetscBool *impl)
    SVDSetTolerances - Sets the tolerance and maximum
    iteration count used by the default SVD convergence testers.
 
-   Logically Collective on SVD
+   Logically Collective on svd
 
    Input Parameters:
 +  svd - the singular value solver context
@@ -92,7 +92,6 @@ PetscErrorCode SVDGetImplicitTranspose(SVD svd,PetscBool *impl)
    Options Database Keys:
 +  -svd_tol <tol> - Sets the convergence tolerance
 -  -svd_max_it <maxits> - Sets the maximum number of iterations allowed
-   (use PETSC_DECIDE to compute an educated guess based on basis and matrix sizes)
 
    Note:
    Use PETSC_DEFAULT for either argument to assign a reasonably good value.
@@ -157,7 +156,7 @@ PetscErrorCode SVDGetTolerances(SVD svd,PetscReal *tol,PetscInt *maxits)
    SVDSetDimensions - Sets the number of singular values to compute
    and the dimension of the subspace.
 
-   Logically Collective on SVD
+   Logically Collective on svd
 
    Input Parameters:
 +  svd - the singular value solver context
@@ -247,7 +246,7 @@ PetscErrorCode SVDGetDimensions(SVD svd,PetscInt *nsv,PetscInt *ncv,PetscInt *mp
     SVDSetWhichSingularTriplets - Specifies which singular triplets are
     to be sought.
 
-    Logically Collective on SVD
+    Logically Collective on svd
 
     Input Parameter:
 .   svd - singular value solver context obtained from SVDCreate()
@@ -320,7 +319,7 @@ PetscErrorCode SVDGetWhichSingularTriplets(SVD svd,SVDWhich *which)
    SVDSetConvergenceTestFunction - Sets a function to compute the error estimate
    used in the convergence test.
 
-   Logically Collective on SVD
+   Logically Collective on svd
 
    Input Parameters:
 +  svd     - singular value solver context obtained from SVDCreate()
@@ -370,7 +369,7 @@ PetscErrorCode SVDSetConvergenceTestFunction(SVD svd,PetscErrorCode (*func)(SVD,
    SVDSetConvergenceTest - Specifies how to compute the error estimate
    used in the convergence test.
 
-   Logically Collective on SVD
+   Logically Collective on svd
 
    Input Parameters:
 +  svd  - singular value solver context obtained from SVDCreate()
@@ -439,7 +438,7 @@ PetscErrorCode SVDGetConvergenceTest(SVD svd,SVDConv *conv)
    SVDSetStoppingTestFunction - Sets a function to decide when to stop the outer
    iteration of the singular value solver.
 
-   Logically Collective on SVD
+   Logically Collective on svd
 
    Input Parameters:
 +  svd     - singular value solver context obtained from SVDCreate()
@@ -492,7 +491,7 @@ PetscErrorCode SVDSetStoppingTestFunction(SVD svd,PetscErrorCode (*func)(SVD,Pet
    SVDSetStoppingTest - Specifies how to decide the termination of the outer
    loop of the singular value solver.
 
-   Logically Collective on SVD
+   Logically Collective on svd
 
    Input Parameters:
 +  svd  - singular value solver context obtained from SVDCreate()
@@ -558,7 +557,7 @@ PetscErrorCode SVDGetStoppingTest(SVD svd,SVDStop *stop)
    SVDMonitorSetFromOptions - Sets a monitor function and viewer appropriate for the type
    indicated by the user.
 
-   Collective on SVD
+   Collective on svd
 
    Input Parameters:
 +  svd      - the singular value solver context
@@ -581,7 +580,7 @@ PetscErrorCode SVDMonitorSetFromOptions(SVD svd,const char name[],const char hel
   PetscViewerAndFormat *vf;
 
   PetscFunctionBegin;
-  ierr = PetscOptionsGetViewer(PetscObjectComm((PetscObject)svd),((PetscObject)svd)->prefix,name,&viewer,&format,&flg);CHKERRQ(ierr);
+  ierr = PetscOptionsGetViewer(PetscObjectComm((PetscObject)svd),((PetscObject)svd)->options,((PetscObject)svd)->prefix,name,&viewer,&format,&flg);CHKERRQ(ierr);
   if (flg) {
     ierr = PetscViewerAndFormatCreate(viewer,format,&vf);CHKERRQ(ierr);
     ierr = PetscObjectDereference((PetscObject)viewer);CHKERRQ(ierr);
@@ -597,7 +596,7 @@ PetscErrorCode SVDMonitorSetFromOptions(SVD svd,const char name[],const char hel
    SVDConvMonitorSetFromOptions - Sets a monitor function and viewer appropriate for the type
    indicated by the user (for monitors that only show iteration numbers of convergence).
 
-   Collective on SVD
+   Collective on svd
 
    Input Parameters:
 +  svd      - the singular value solver context
@@ -619,7 +618,7 @@ PetscErrorCode SVDConvMonitorSetFromOptions(SVD svd,const char name[],const char
   SlepcConvMonitor  ctx;
 
   PetscFunctionBegin;
-  ierr = PetscOptionsGetViewer(PetscObjectComm((PetscObject)svd),((PetscObject)svd)->prefix,name,&viewer,&format,&flg);CHKERRQ(ierr);
+  ierr = PetscOptionsGetViewer(PetscObjectComm((PetscObject)svd),((PetscObject)svd)->options,((PetscObject)svd)->prefix,name,&viewer,&format,&flg);CHKERRQ(ierr);
   if (flg) {
     ierr = SlepcConvMonitorCreate(viewer,format,&ctx);CHKERRQ(ierr);
     ierr = PetscObjectDereference((PetscObject)viewer);CHKERRQ(ierr);
@@ -633,7 +632,7 @@ PetscErrorCode SVDConvMonitorSetFromOptions(SVD svd,const char name[],const char
    This routine must be called before SVDSetUp() if the user is to be
    allowed to set the solver type.
 
-   Collective on SVD
+   Collective on svd
 
    Input Parameters:
 .  svd - the singular value solver context
@@ -742,8 +741,9 @@ PetscErrorCode SVDSetFromOptions(SVD svd)
     ierr = PetscObjectProcessOptionsHandlers(PetscOptionsObject,(PetscObject)svd);CHKERRQ(ierr);
   ierr = PetscOptionsEnd();CHKERRQ(ierr);
 
-  if (!svd->V) { ierr = SVDGetBV(svd,&svd->V,&svd->U);CHKERRQ(ierr); }
+  if (!svd->V) { ierr = SVDGetBV(svd,&svd->V,NULL);CHKERRQ(ierr); }
   ierr = BVSetFromOptions(svd->V);CHKERRQ(ierr);
+  if (!svd->U) { ierr = SVDGetBV(svd,NULL,&svd->U);CHKERRQ(ierr); }
   ierr = BVSetFromOptions(svd->U);CHKERRQ(ierr);
   if (!svd->ds) { ierr = SVDGetDS(svd,&svd->ds);CHKERRQ(ierr); }
   ierr = DSSetFromOptions(svd->ds);CHKERRQ(ierr);
@@ -754,7 +754,7 @@ PetscErrorCode SVDSetFromOptions(SVD svd)
    SVDSetTrackAll - Specifies if the solver must compute the residual norm of all
    approximate singular value or not.
 
-   Logically Collective on SVD
+   Logically Collective on svd
 
    Input Parameters:
 +  svd      - the singular value solver context
@@ -802,7 +802,7 @@ PetscErrorCode SVDGetTrackAll(SVD svd,PetscBool *trackall)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
-  PetscValidPointer(trackall,2);
+  PetscValidBoolPointer(trackall,2);
   *trackall = svd->trackall;
   PetscFunctionReturn(0);
 }
@@ -812,7 +812,7 @@ PetscErrorCode SVDGetTrackAll(SVD svd,PetscBool *trackall)
    SVDSetOptionsPrefix - Sets the prefix used for searching for all
    SVD options in the database.
 
-   Logically Collective on SVD
+   Logically Collective on svd
 
    Input Parameters:
 +  svd - the singular value solver context
@@ -853,7 +853,7 @@ PetscErrorCode SVDSetOptionsPrefix(SVD svd,const char *prefix)
    SVDAppendOptionsPrefix - Appends to the prefix used for searching for all
    SVD options in the database.
 
-   Logically Collective on SVD
+   Logically Collective on svd
 
    Input Parameters:
 +  svd - the singular value solver context
