@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2018, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -54,9 +54,10 @@ PetscErrorCode DSFinalizePackage(void)
 @*/
 PetscErrorCode DSInitializePackage()
 {
-  char             logList[256];
+  char           logList[256];
   PetscBool      opt,pkg;
-  PetscErrorCode   ierr;
+  PetscClassId   classids[1];
+  PetscErrorCode ierr;
 
   PetscFunctionBegin;
   if (DSPackageInitialized) PetscFunctionReturn(0);
@@ -70,12 +71,9 @@ PetscErrorCode DSInitializePackage()
   ierr = PetscLogEventRegister("DSVectors",DS_CLASSID,&DS_Vectors);CHKERRQ(ierr);
   ierr = PetscLogEventRegister("DSSynchronize",DS_CLASSID,&DS_Synchronize);CHKERRQ(ierr);
   ierr = PetscLogEventRegister("DSOther",DS_CLASSID,&DS_Other);CHKERRQ(ierr);
-  /* Process info exclusions */
-  ierr = PetscOptionsGetString(NULL,NULL,"-info_exclude",logList,sizeof(logList),&opt);CHKERRQ(ierr);
-  if (opt) {
-    ierr = PetscStrInList("ds",logList,',',&pkg);CHKERRQ(ierr);
-    if (pkg) { ierr = PetscInfoDeactivateClass(DS_CLASSID);CHKERRQ(ierr); }
-  }
+  /* Process Info */
+  classids[0] = DS_CLASSID;
+  ierr = PetscInfoProcessClass("ds",1,&classids[0]);CHKERRQ(ierr);
   /* Process summary exclusions */
   ierr = PetscOptionsGetString(NULL,NULL,"-log_exclude",logList,sizeof(logList),&opt);CHKERRQ(ierr);
   if (opt) {
@@ -90,7 +88,7 @@ PetscErrorCode DSInitializePackage()
 /*@
    DSCreate - Creates a DS context.
 
-   Collective on MPI_Comm
+   Collective
 
    Input Parameter:
 .  comm - MPI communicator
@@ -140,6 +138,7 @@ PetscErrorCode DSCreate(MPI_Comm comm,DS *newds)
   }
   ds->perm          = NULL;
   ds->data          = NULL;
+  ds->scset         = PETSC_FALSE;
   ds->work          = NULL;
   ds->rwork         = NULL;
   ds->iwork         = NULL;
@@ -155,7 +154,7 @@ PetscErrorCode DSCreate(MPI_Comm comm,DS *newds)
    DSSetOptionsPrefix - Sets the prefix used for searching for all
    DS options in the database.
 
-   Logically Collective on DS
+   Logically Collective on ds
 
    Input Parameters:
 +  ds - the direct solver context
@@ -184,7 +183,7 @@ PetscErrorCode DSSetOptionsPrefix(DS ds,const char *prefix)
    DSAppendOptionsPrefix - Appends to the prefix used for searching for all
    DS options in the database.
 
-   Logically Collective on DS
+   Logically Collective on ds
 
    Input Parameters:
 +  ds - the direct solver context
@@ -242,7 +241,7 @@ PetscErrorCode DSGetOptionsPrefix(DS ds,const char *prefix[])
 /*@C
    DSSetType - Selects the type for the DS object.
 
-   Logically Collective on DS
+   Logically Collective on ds
 
    Input Parameter:
 +  ds   - the direct solver context
@@ -298,10 +297,51 @@ PetscErrorCode DSGetType(DS ds,DSType *type)
   PetscFunctionReturn(0);
 }
 
+/*@
+   DSDuplicate - Creates a new direct solver object with the same options as
+   an existing one.
+
+   Collective on ds
+
+   Input Parameter:
+.  ds - direct solver context
+
+   Output Parameter:
+.  dsnew - location to put the new DS
+
+   Notes:
+   DSDuplicate() DOES NOT COPY the matrices, and the new DS does not even have
+   internal arrays allocated. Use DSAllocate() to use the new DS.
+
+   The sorting criterion options are not copied, see DSSetSlepcSC().
+
+   Level: intermediate
+
+.seealso: DSCreate(), DSAllocate(), DSSetSlepcSC()
+@*/
+PetscErrorCode DSDuplicate(DS ds,DS *dsnew)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(ds,DS_CLASSID,1);
+  PetscValidType(ds,1);
+  PetscValidPointer(dsnew,2);
+  ierr = DSCreate(PetscObjectComm((PetscObject)ds),dsnew);CHKERRQ(ierr);
+  ierr = DSSetType(*dsnew,((PetscObject)ds)->type_name);CHKERRQ(ierr);
+  (*dsnew)->method   = ds->method;
+  (*dsnew)->compact  = ds->compact;
+  (*dsnew)->refined  = ds->refined;
+  (*dsnew)->extrarow = ds->extrarow;
+  (*dsnew)->bs       = ds->bs;
+  (*dsnew)->pmode    = ds->pmode;
+  PetscFunctionReturn(0);
+}
+
 /*@
    DSSetMethod - Selects the method to be used to solve the problem.
 
-   Logically Collective on DS
+   Logically Collective on ds
 
    Input Parameter:
 +  ds   - the direct solver context
@@ -344,7 +384,7 @@ PetscErrorCode DSGetMethod(DS ds,PetscInt *meth)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ds,DS_CLASSID,1);
-  PetscValidPointer(meth,2);
+  PetscValidIntPointer(meth,2);
   *meth = ds->method;
   PetscFunctionReturn(0);
 }
@@ -352,7 +392,7 @@ PetscErrorCode DSGetMethod(DS ds,PetscInt *meth)
 /*@
    DSSetParallel - Selects the mode of operation in parallel runs.
 
-   Logically Collective on DS
+   Logically Collective on ds
 
    Input Parameter:
 +  ds    - the direct solver context
@@ -412,7 +452,7 @@ PetscErrorCode DSGetParallel(DS ds,DSParallelType *pmode)
 /*@
    DSSetCompact - Switch to compact storage of matrices.
 
-   Logically Collective on DS
+   Logically Collective on ds
 
    Input Parameter:
 +  ds   - the direct solver context
@@ -458,7 +498,7 @@ PetscErrorCode DSGetCompact(DS ds,PetscBool *comp)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ds,DS_CLASSID,1);
-  PetscValidPointer(comp,2);
+  PetscValidBoolPointer(comp,2);
   *comp = ds->compact;
   PetscFunctionReturn(0);
 }
@@ -467,7 +507,7 @@ PetscErrorCode DSGetCompact(DS ds,PetscBool *comp)
    DSSetExtraRow - Sets a flag to indicate that the matrix has one extra
    row.
 
-   Logically Collective on DS
+   Logically Collective on ds
 
    Input Parameter:
 +  ds  - the direct solver context
@@ -514,7 +554,7 @@ PetscErrorCode DSGetExtraRow(DS ds,PetscBool *ext)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ds,DS_CLASSID,1);
-  PetscValidPointer(ext,2);
+  PetscValidBoolPointer(ext,2);
   *ext = ds->extrarow;
   PetscFunctionReturn(0);
 }
@@ -523,7 +563,7 @@ PetscErrorCode DSGetExtraRow(DS ds,PetscBool *ext)
    DSSetRefined - Sets a flag to indicate that refined vectors must be
    computed.
 
-   Logically Collective on DS
+   Logically Collective on ds
 
    Input Parameter:
 +  ds  - the direct solver context
@@ -571,7 +611,7 @@ PetscErrorCode DSGetRefined(DS ds,PetscBool *ref)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ds,DS_CLASSID,1);
-  PetscValidPointer(ref,2);
+  PetscValidBoolPointer(ref,2);
   *ref = ds->refined;
   PetscFunctionReturn(0);
 }
@@ -579,7 +619,7 @@ PetscErrorCode DSGetRefined(DS ds,PetscBool *ref)
 /*@
    DSSetBlockSize - Sets the block size.
 
-   Logically Collective on DS
+   Logically Collective on ds
 
    Input Parameter:
 +  ds - the direct solver context
@@ -621,7 +661,7 @@ PetscErrorCode DSGetBlockSize(DS ds,PetscInt *bs)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ds,DS_CLASSID,1);
-  PetscValidPointer(bs,2);
+  PetscValidIntPointer(bs,2);
   *bs = ds->bs;
   PetscFunctionReturn(0);
 }
@@ -646,10 +686,11 @@ PetscErrorCode DSSetSlepcSC(DS ds,SlepcSC sc)
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ds,DS_CLASSID,1);
   PetscValidPointer(sc,2);
-  if (ds->sc) {
+  if (ds->sc && !ds->scset) {
     ierr = PetscFree(ds->sc);CHKERRQ(ierr);
   }
-  ds->sc = sc;
+  ds->sc    = sc;
+  ds->scset = PETSC_TRUE;
   PetscFunctionReturn(0);
 }
 
@@ -685,7 +726,7 @@ PetscErrorCode DSGetSlepcSC(DS ds,SlepcSC *sc)
 /*@
    DSSetFromOptions - Sets DS options from the options database.
 
-   Collective on DS
+   Collective on ds
 
    Input Parameters:
 .  ds - the direct solver context
@@ -728,7 +769,7 @@ PetscErrorCode DSSetFromOptions(DS ds)
 /*@C
    DSView - Prints the DS data structure.
 
-   Collective on DS
+   Collective on ds
 
    Input Parameters:
 +  ds - the direct solver context
@@ -752,27 +793,26 @@ PetscErrorCode DSSetFromOptions(DS ds)
 PetscErrorCode DSView(DS ds,PetscViewer viewer)
 {
   PetscBool         isascii,issvd;
-  PetscInt          tabs;
   PetscViewerFormat format;
   PetscErrorCode    ierr;
   PetscMPIInt       size;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ds,DS_CLASSID,1);
-  if (!viewer) viewer = PETSC_VIEWER_STDOUT_(PetscObjectComm((PetscObject)ds));
+  if (!viewer) {
+    ierr = PetscViewerASCIIGetStdout(PetscObjectComm((PetscObject)ds),&viewer);CHKERRQ(ierr);
+  }
   PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,2);
   PetscCheckSameComm(ds,1,viewer,2);
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
   if (isascii) {
-    ierr = PetscViewerASCIIGetTab(viewer,&tabs);CHKERRQ(ierr);
-    ierr = PetscViewerASCIISetTab(viewer,((PetscObject)ds)->tablevel);CHKERRQ(ierr);
     ierr = PetscViewerGetFormat(viewer,&format);CHKERRQ(ierr);
     ierr = PetscObjectPrintClassNamePrefixType((PetscObject)ds,viewer);CHKERRQ(ierr);
     ierr = MPI_Comm_size(PetscObjectComm((PetscObject)ds),&size);CHKERRQ(ierr);
-    if (size>1) {
-      ierr = PetscViewerASCIIPrintf(viewer,"  parallel operation mode: %s\n",DSParallelTypes[ds->pmode]);CHKERRQ(ierr);
-    }
     if (format == PETSC_VIEWER_ASCII_INFO_DETAIL) {
+      if (size>1) {
+        ierr = PetscViewerASCIIPrintf(viewer,"  parallel operation mode: %s\n",DSParallelTypes[ds->pmode]);CHKERRQ(ierr);
+      }
       ierr = PetscViewerASCIIPrintf(viewer,"  current state: %s\n",DSStateTypes[ds->state]);CHKERRQ(ierr);
       ierr = PetscObjectTypeCompare((PetscObject)ds,DSSVD,&issvd);CHKERRQ(ierr);
       if (issvd) {
@@ -792,15 +832,38 @@ PetscErrorCode DSView(DS ds,PetscViewer viewer)
       ierr = (*ds->ops->view)(ds,viewer);CHKERRQ(ierr);
       ierr = PetscViewerASCIIPopTab(viewer);CHKERRQ(ierr);
     }
-    ierr = PetscViewerASCIISetTab(viewer,tabs);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
+/*@C
+   DSViewFromOptions - View from options
+
+   Collective on DS
+
+   Input Parameters:
++  ds   - the direct solver context
+.  obj  - optional object
+-  name - command line option
+
+   Level: intermediate
+
+.seealso: DSView(), DSCreate()
+@*/
+PetscErrorCode DSViewFromOptions(DS ds,PetscObject obj,const char name[])
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(ds,DS_CLASSID,1);
+  ierr = PetscObjectViewFromOptions((PetscObject)ds,obj,name);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
 /*@
    DSAllocate - Allocates memory for internal storage or matrices in DS.
 
-   Logically Collective on DS
+   Logically Collective on ds
 
    Input Parameters:
 +  ds - the direct solver context
@@ -835,7 +898,7 @@ PetscErrorCode DSAllocate(DS ds,PetscInt ld)
 /*@
    DSReset - Resets the DS context to the initial state.
 
-   Collective on DS
+   Collective on ds
 
    Input Parameter:
 .  ds - the direct solver context
@@ -874,7 +937,7 @@ PetscErrorCode DSReset(DS ds)
 /*@
    DSDestroy - Destroys DS context that was created with DSCreate().
 
-   Collective on DS
+   Collective on ds
 
    Input Parameter:
 .  ds - the direct solver context
@@ -896,7 +959,7 @@ PetscErrorCode DSDestroy(DS *ds)
   ierr = PetscFree((*ds)->work);CHKERRQ(ierr);
   ierr = PetscFree((*ds)->rwork);CHKERRQ(ierr);
   ierr = PetscFree((*ds)->iwork);CHKERRQ(ierr);
-  ierr = PetscFree((*ds)->sc);CHKERRQ(ierr);
+  if (!(*ds)->scset) { ierr = PetscFree((*ds)->sc);CHKERRQ(ierr); }
   ierr = PetscHeaderDestroy(ds);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -923,18 +986,19 @@ PetscErrorCode DSRegister(const char *name,PetscErrorCode (*function)(DS))
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
+  ierr = DSInitializePackage();CHKERRQ(ierr);
   ierr = PetscFunctionListAdd(&DSList,name,function);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-PETSC_EXTERN PetscErrorCode DSCreate_HEP(DS);
-PETSC_EXTERN PetscErrorCode DSCreate_NHEP(DS);
-PETSC_EXTERN PetscErrorCode DSCreate_GHEP(DS);
-PETSC_EXTERN PetscErrorCode DSCreate_GHIEP(DS);
-PETSC_EXTERN PetscErrorCode DSCreate_GNHEP(DS);
-PETSC_EXTERN PetscErrorCode DSCreate_SVD(DS);
-PETSC_EXTERN PetscErrorCode DSCreate_PEP(DS);
-PETSC_EXTERN PetscErrorCode DSCreate_NEP(DS);
+SLEPC_EXTERN PetscErrorCode DSCreate_HEP(DS);
+SLEPC_EXTERN PetscErrorCode DSCreate_NHEP(DS);
+SLEPC_EXTERN PetscErrorCode DSCreate_GHEP(DS);
+SLEPC_EXTERN PetscErrorCode DSCreate_GHIEP(DS);
+SLEPC_EXTERN PetscErrorCode DSCreate_GNHEP(DS);
+SLEPC_EXTERN PetscErrorCode DSCreate_SVD(DS);
+SLEPC_EXTERN PetscErrorCode DSCreate_PEP(DS);
+SLEPC_EXTERN PetscErrorCode DSCreate_NEP(DS);
 
 /*@C
    DSRegisterAll - Registers all of the direct solvers in the DS package.
