@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -33,7 +33,7 @@ PetscErrorCode DSGetLeadingDimension(DS ds,PetscInt *ld)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ds,DS_CLASSID,1);
-  PetscValidPointer(ld,2);
+  PetscValidIntPointer(ld,2);
   *ld = ds->ld;
   PetscFunctionReturn(0);
 }
@@ -41,7 +41,7 @@ PetscErrorCode DSGetLeadingDimension(DS ds,PetscInt *ld)
 /*@
    DSSetState - Change the state of the DS object.
 
-   Logically Collective on DS
+   Logically Collective on ds
 
    Input Parameters:
 +  ds    - the direct solver context
@@ -108,7 +108,7 @@ PetscErrorCode DSGetState(DS ds,DSStateType *state)
 /*@
    DSSetDimensions - Resize the matrices in the DS object.
 
-   Logically Collective on DS
+   Logically Collective on ds
 
    Input Parameters:
 +  ds - the direct solver context
@@ -205,7 +205,7 @@ PetscErrorCode DSGetDimensions(DS ds,PetscInt *n,PetscInt *m,PetscInt *l,PetscIn
 /*@
    DSTruncate - Truncates the system represented in the DS object.
 
-   Logically Collective on DS
+   Logically Collective on ds
 
    Input Parameters:
 +  ds - the direct solver context
@@ -310,7 +310,7 @@ PetscErrorCode DSMatIsHermitian(DS ds,DSMatType t,PetscBool *flg)
   PetscValidHeaderSpecific(ds,DS_CLASSID,1);
   PetscValidType(ds,1);
   DSCheckValidMat(ds,t,2);
-  PetscValidPointer(flg,3);
+  PetscValidBoolPointer(flg,3);
   if (ds->ops->hermitian) {
     ierr = (*ds->ops->hermitian)(ds,t,flg);CHKERRQ(ierr);
   } else *flg = PETSC_FALSE;
@@ -381,7 +381,7 @@ PetscErrorCode DSGetMat(DS ds,DSMatType m,Mat *A)
   M  = ds->mat[m];
   ierr = MatDenseGetArray(*A,&pA);CHKERRQ(ierr);
   for (j=0;j<cols;j++) {
-    ierr = PetscMemcpy(pA+j*rows,M+j*ds->ld,rows*sizeof(PetscScalar));CHKERRQ(ierr);
+    ierr = PetscArraycpy(pA+j*rows,M+j*ds->ld,rows);CHKERRQ(ierr);
   }
   ierr = MatDenseRestoreArray(*A,&pA);CHKERRQ(ierr);
   PetscFunctionReturn(0);
@@ -427,7 +427,7 @@ PetscErrorCode DSRestoreMat(DS ds,DSMatType m,Mat *A)
   M  = ds->mat[m];
   ierr = MatDenseGetArray(*A,&pA);CHKERRQ(ierr);
   for (j=0;j<cols;j++) {
-    ierr = PetscMemcpy(M+j*ds->ld,pA+j*rows,rows*sizeof(PetscScalar));CHKERRQ(ierr);
+    ierr = PetscArraycpy(M+j*ds->ld,pA+j*rows,rows);CHKERRQ(ierr);
   }
   ierr = MatDenseRestoreArray(*A,&pA);CHKERRQ(ierr);
   ierr = MatDestroy(A);CHKERRQ(ierr);
@@ -555,7 +555,7 @@ PetscErrorCode DSRestoreArrayReal(DS ds,DSMatType m,PetscReal *a[])
 /*@
    DSSolve - Solves the problem.
 
-   Logically Collective on DS
+   Logically Collective on ds
 
    Input Parameters:
 +  ds   - the direct solver context
@@ -578,7 +578,7 @@ PetscErrorCode DSSolve(DS ds,PetscScalar eigr[],PetscScalar eigi[])
   PetscValidHeaderSpecific(ds,DS_CLASSID,1);
   PetscValidType(ds,1);
   DSCheckAlloc(ds,1);
-  PetscValidPointer(eigr,2);
+  PetscValidScalarPointer(eigr,2);
   if (ds->state>=DS_STATE_CONDENSED) PetscFunctionReturn(0);
   if (!ds->ops->solve[ds->method]) SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_OUTOFRANGE,"The specified method number does not exist for this DS");
   ierr = PetscLogEventBegin(DS_Solve,ds,0,0,0);CHKERRQ(ierr);
@@ -595,17 +595,17 @@ PetscErrorCode DSSolve(DS ds,PetscScalar eigr[],PetscScalar eigi[])
    DSSort - Sorts the result of DSSolve() according to a given sorting
    criterion.
 
-   Logically Collective on DS
+   Logically Collective on ds
 
    Input Parameters:
 +  ds   - the direct solver context
-.  eigr - array containing the computed eigenvalues (real part)
-.  eigi - array containing the computed eigenvalues (imaginary part)
 .  rr   - (optional) array containing auxiliary values (real part)
 -  ri   - (optional) array containing auxiliary values (imaginary part)
 
-   Input/Output Parameter:
-.  k    - (optional) number of elements in the leading group
+   Input/Output Parameters:
++  eigr - array containing the computed eigenvalues (real part)
+.  eigi - array containing the computed eigenvalues (imaginary part)
+-  k    - (optional) number of elements in the leading group
 
    Notes:
    This routine sorts the arrays provided in eigr and eigi, and also
@@ -622,7 +622,7 @@ PetscErrorCode DSSolve(DS ds,PetscScalar eigr[],PetscScalar eigi[])
 
    Level: intermediate
 
-.seealso: DSSolve(), DSSetSlepcSC()
+.seealso: DSSolve(), DSSetSlepcSC(), DSSortWithPermutation()
 @*/
 PetscErrorCode DSSort(DS ds,PetscScalar *eigr,PetscScalar *eigi,PetscScalar *rr,PetscScalar *ri,PetscInt *k)
 {
@@ -633,8 +633,8 @@ PetscErrorCode DSSort(DS ds,PetscScalar *eigr,PetscScalar *eigi,PetscScalar *rr,
   PetscValidHeaderSpecific(ds,DS_CLASSID,1);
   PetscValidType(ds,1);
   DSCheckSolved(ds,1);
-  PetscValidPointer(eigr,2);
-  if (rr) PetscValidPointer(rr,4);
+  PetscValidScalarPointer(eigr,2);
+  if (rr) PetscValidScalarPointer(rr,4);
   if (ds->state==DS_STATE_TRUNCATED) SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_ORDER,"Cannot sort a truncated DS");
   if (!ds->ops->sort) SETERRQ1(PetscObjectComm((PetscObject)ds),PETSC_ERR_SUP,"DS type %s",((PetscObject)ds)->type_name);
   if (!ds->sc) SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_ORDER,"Must provide a sorting criterion first");
@@ -650,11 +650,56 @@ PetscErrorCode DSSort(DS ds,PetscScalar *eigr,PetscScalar *eigi,PetscScalar *rr,
   PetscFunctionReturn(0);
 }
 
+/*@C
+   DSSortWithPermutation - Reorders the result of DSSolve() according to a given
+   permutation.
+
+   Logically Collective on ds
+
+   Input Parameters:
++  ds   - the direct solver context
+-  perm - permutation that indicates the new ordering
+
+   Input/Output Parameters:
++  eigr - array with the reordered eigenvalues (real part)
+-  eigi - array with the reordered eigenvalues (imaginary part)
+
+   Notes:
+   This routine reorders the arrays provided in eigr and eigi, and also the dense
+   system stored inside ds (assumed to be in condensed form). There is no sorting
+   criterion, as opposed to DSSort(). Instead, the new ordering is given in argument perm.
+
+   Level: advanced
+
+.seealso: DSSolve(), DSSort()
+@*/
+PetscErrorCode DSSortWithPermutation(DS ds,PetscInt *perm,PetscScalar *eigr,PetscScalar *eigi)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(ds,DS_CLASSID,1);
+  PetscValidType(ds,1);
+  DSCheckSolved(ds,1);
+  PetscValidIntPointer(perm,2);
+  PetscValidScalarPointer(eigr,3);
+  if (ds->state==DS_STATE_TRUNCATED) SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_ORDER,"Cannot sort a truncated DS");
+  if (!ds->ops->sortperm) SETERRQ1(PetscObjectComm((PetscObject)ds),PETSC_ERR_SUP,"DS type %s",((PetscObject)ds)->type_name);
+
+  ierr = PetscLogEventBegin(DS_Other,ds,0,0,0);CHKERRQ(ierr);
+  ierr = PetscFPTrapPush(PETSC_FP_TRAP_OFF);CHKERRQ(ierr);
+  ierr = (*ds->ops->sortperm)(ds,perm,eigr,eigi);CHKERRQ(ierr);
+  ierr = PetscFPTrapPop();CHKERRQ(ierr);
+  ierr = PetscLogEventEnd(DS_Other,ds,0,0,0);CHKERRQ(ierr);
+  ierr = PetscObjectStateIncrease((PetscObject)ds);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
 /*@
    DSSynchronize - Make sure that all processes have the same data, performing
    communication if necessary.
 
-   Collective on DS
+   Collective on ds
 
    Input Parameter:
 +  ds   - the direct solver context
@@ -705,7 +750,7 @@ PetscErrorCode DSSynchronize(DS ds,PetscScalar eigr[],PetscScalar eigi[])
    DSVectors - Compute vectors associated to the dense system such
    as eigenvectors.
 
-   Logically Collective on DS
+   Logically Collective on ds
 
    Input Parameters:
 +  ds  - the direct solver context
@@ -813,7 +858,7 @@ PetscErrorCode DSCond(DS ds,PetscReal *cond)
   PetscValidHeaderSpecific(ds,DS_CLASSID,1);
   PetscValidType(ds,1);
   DSCheckAlloc(ds,1);
-  PetscValidPointer(cond,2);
+  PetscValidRealPointer(cond,2);
   if (!ds->ops->cond) SETERRQ1(PetscObjectComm((PetscObject)ds),PETSC_ERR_SUP,"DS type %s",((PetscObject)ds)->type_name);
   ierr = PetscLogEventBegin(DS_Other,ds,0,0,0);CHKERRQ(ierr);
   ierr = PetscFPTrapPush(PETSC_FP_TRAP_OFF);CHKERRQ(ierr);
@@ -826,7 +871,7 @@ PetscErrorCode DSCond(DS ds,PetscReal *cond)
 /*@C
    DSTranslateHarmonic - Computes a translation of the dense system.
 
-   Logically Collective on DS
+   Logically Collective on ds
 
    Input Parameters:
 +  ds      - the direct solver context
@@ -876,7 +921,7 @@ PetscErrorCode DSTranslateHarmonic(DS ds,PetscScalar tau,PetscReal beta,PetscBoo
    DSTranslateRKS - Computes a modification of the dense system corresponding
    to an update of the shift in a rational Krylov method.
 
-   Logically Collective on DS
+   Logically Collective on ds
 
    Input Parameters:
 +  ds    - the direct solver context
@@ -977,9 +1022,9 @@ PetscErrorCode DSCopyMat(DS ds,DSMatType m,PetscInt mr,PetscInt mc,Mat A,PetscIn
   ierr = MatDenseGetArray(A,&pA);CHKERRQ(ierr);
   for (j=0;j<cols;j++) {
     if (out) {
-      ierr = PetscMemcpy(pA+(Ac+j)*arows+Ar,M+(mc+j)*ds->ld+mr,rows*sizeof(PetscScalar));CHKERRQ(ierr);
+      ierr = PetscArraycpy(pA+(Ac+j)*arows+Ar,M+(mc+j)*ds->ld+mr,rows);CHKERRQ(ierr);
     } else {
-      ierr = PetscMemcpy(M+(mc+j)*ds->ld+mr,pA+(Ac+j)*arows+Ar,rows*sizeof(PetscScalar));CHKERRQ(ierr);
+      ierr = PetscArraycpy(M+(mc+j)*ds->ld+mr,pA+(Ac+j)*arows+Ar,rows);CHKERRQ(ierr);
     }
   }
   ierr = MatDenseRestoreArray(A,&pA);CHKERRQ(ierr);
